@@ -3,7 +3,7 @@
  *                    2000 Wim Taymans <wtay@chello.be>
  *                    2001 Bastien Nocera <hadess@hadess.net>
  *                    2003 Colin Walters <walters@verbum.org>
- *                    2005 Tim-Philipp Müller <tim centricular net>
+ *                    2005 Tim-Philipp M??ller <tim centricular net>
  *
  * gstgnomevfssink.c: 
  *
@@ -25,27 +25,36 @@
 
 /**
  * SECTION:element-gnomevfssink
+ * @short_description: Write a stream to a GnomeVFS URI
  * @see_also: #GstFileSink, #GstGnomeVFSSrc
  *
+ * <refsect2>
+ * <para>
  * This plugin writes incoming data to a local or remote location specified
  * by an URI. This location can be specified using any protocol supported by
  * the GnomeVFS library. Common protocols are 'file', 'ftp', or 'smb'.
- *
- * Applications can connect to the #GstGnomeVFSSink::allow-overwrite signal to
- * receive a callback when an existing file will be overwritten. The return
- * value of the signal will determine if gnomevfssink will overwrite the
- * resource or abort with an error.
- *
- * <refsect2>
- * <title>Example launch lines</title>
- * |[
+ * </para>
+ * <para>
+ * Example pipeline:
+ * <programlisting>
  * gst-launch -v filesrc location=input.xyz ! gnomevfssink location=file:///home/joe/out.xyz
- * ]| The above pipeline will simply copy a local file. Instead of gnomevfssink,
+ * </programlisting>
+ * The above pipeline will simply copy a local file. Instead of gnomevfssink,
  * we could just as well have used the filesink element here.
- * |[
+ * </para>
+ * <para>
+ * Another example pipeline:
+ * <programlisting>
  * gst-launch -v filesrc location=foo.mp3 ! mad ! flacenc ! gnomevfssink location=smb://othercomputer/foo.flac
- * ]| The above pipeline will re-encode an mp3 file into FLAC format and store
+ * </programlisting>
+ * The above pipeline will re-encode an mp3 file into FLAC format and store
  * it on a remote host using the Samba protocol.
+ * </para>
+ * <para>
+ * Applications can connect to the allow-overwrite signal to receive a callback when an
+ * existing file will be overwritten. The return value of the signal will determine if
+ * gnomevfssink will overwrite the resource or abort with an error.
+ * </para>
  * </refsect2>
  *
  * Last reviewed on 2006-02-28 (0.10.4)
@@ -64,6 +73,12 @@
 #include <string.h>
 #include <errno.h>
 
+static const GstElementDetails gst_gnome_vfs_sink_details =
+GST_ELEMENT_DETAILS ("GnomeVFS Sink",
+    "Sink/File",
+    "Write a stream to a GnomeVFS URI",
+    "Bastien Nocera <hadess@hadess.net>");
+
 enum
 {
   SIGNAL_ERASE_ASK,
@@ -131,11 +146,10 @@ gst_gnome_vfs_sink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class, &sinktemplate);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sinktemplate));
 
-  gst_element_class_set_details_simple (element_class,
-      "GnomeVFS Sink", "Sink/File",
-      "Write a stream to a GnomeVFS URI", "Bastien Nocera <hadess@hadess.net>");
+  gst_element_class_set_details (element_class, &gst_gnome_vfs_sink_details);
 }
 
 static gboolean
@@ -167,15 +181,14 @@ gst_gnome_vfs_sink_class_init (GstGnomeVFSSinkClass * klass)
 
   g_object_class_install_property (gobject_class, ARG_LOCATION,
       g_param_spec_string ("location", "File Location",
-          "Location of the file to write", NULL,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          "Location of the file to write", NULL, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_URI,
       g_param_spec_boxed ("uri", "GnomeVFSURI", "URI for GnomeVFS",
-          GST_TYPE_GNOME_VFS_URI, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          GST_TYPE_GNOME_VFS_URI, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_HANDLE,
-      g_param_spec_boxed ("handle", "GnomeVFSHandle", "Handle for GnomeVFS",
-          GST_TYPE_GNOME_VFS_HANDLE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+      g_param_spec_boxed ("handle",
+          "GnomeVFSHandle", "Handle for GnomeVFS",
+          GST_TYPE_GNOME_VFS_HANDLE, G_PARAM_READWRITE));
 
   /**
    * GstGnomeVFSSink::allow-overwrite
@@ -229,7 +242,7 @@ gst_gnome_vfs_sink_init (GstGnomeVFSSink * sink, GstGnomeVFSSinkClass * klass)
   sink->uri_name = NULL;
   sink->handle = NULL;
   sink->own_handle = FALSE;
-  sink->current_pos = 0;
+  sink->data_written = 0;
 
   GST_BASE_SINK (sink)->sync = FALSE;
 }
@@ -385,7 +398,7 @@ gst_gnome_vfs_sink_open_file (GstGnomeVFSSink * sink)
     sink->own_handle = FALSE;
   }
 
-  sink->current_pos = 0;
+  sink->data_written = 0;
 
   return TRUE;
 }
@@ -435,7 +448,6 @@ static gboolean
 gst_gnome_vfs_sink_handle_event (GstBaseSink * basesink, GstEvent * event)
 {
   GstGnomeVFSSink *sink;
-  gboolean ret = TRUE;
 
   sink = GST_GNOME_VFS_SINK (basesink);
 
@@ -462,9 +474,6 @@ gst_gnome_vfs_sink_handle_event (GstBaseSink * basesink, GstEvent * event)
       if (res != GNOME_VFS_OK) {
         GST_ERROR_OBJECT (sink, "Failed to seek to offset %"
             G_GINT64_FORMAT ": %s", offset, gnome_vfs_result_to_string (res));
-        ret = FALSE;
-      } else {
-        sink->current_pos = offset;
       }
 
       break;
@@ -472,14 +481,14 @@ gst_gnome_vfs_sink_handle_event (GstBaseSink * basesink, GstEvent * event)
 
     case GST_EVENT_FLUSH_START:
     case GST_EVENT_EOS:{
-      /* No need to flush with GnomeVfs */
+      /* how does Gnome-VFS flush? Do we need to flush? */
       break;
     }
     default:
       break;
   }
 
-  return ret;
+  return TRUE;
 }
 
 static gboolean
@@ -496,7 +505,7 @@ gst_gnome_vfs_sink_query (GstPad * pad, GstQuery * query)
       switch (format) {
         case GST_FORMAT_DEFAULT:
         case GST_FORMAT_BYTES:
-          gst_query_set_position (query, GST_FORMAT_BYTES, sink->current_pos);
+          gst_query_set_position (query, GST_FORMAT_BYTES, sink->data_written);
           return TRUE;
         default:
           return FALSE;
@@ -506,10 +515,6 @@ gst_gnome_vfs_sink_query (GstPad * pad, GstQuery * query)
       gst_query_set_formats (query, 2, GST_FORMAT_DEFAULT, GST_FORMAT_BYTES);
       return TRUE;
 
-    case GST_QUERY_URI:
-      gst_query_set_uri (query, sink->uri_name);
-      return TRUE;
-
     default:
       return gst_pad_query_default (pad, query);
   }
@@ -522,12 +527,13 @@ gst_gnome_vfs_sink_render (GstBaseSink * basesink, GstBuffer * buf)
   GstGnomeVFSSink *sink;
   GnomeVFSResult result;
   GstFlowReturn ret;
+  guint64 back_pending = 0;
 
   sink = GST_GNOME_VFS_SINK (basesink);
 
   if (gnome_vfs_tell (sink->handle, &cur_pos) == GNOME_VFS_OK) {
-    /* bring up to date with current position for proper reporting */
-    sink->current_pos = cur_pos;
+    if (cur_pos < sink->data_written)
+      back_pending = sink->data_written - cur_pos;
   }
 
   result = gnome_vfs_write (sink->handle, GST_BUFFER_DATA (buf),
@@ -545,7 +551,7 @@ gst_gnome_vfs_sink_render (GstBaseSink * basesink, GstBuffer * buf)
             GST_BUFFER_SIZE (buf), written);
       }
 
-      sink->current_pos += GST_BUFFER_SIZE (buf);
+      sink->data_written += GST_BUFFER_SIZE (buf) - back_pending;
       ret = GST_FLOW_OK;
       break;
     }
@@ -572,12 +578,12 @@ gst_gnome_vfs_sink_render (GstBaseSink * basesink, GstBuffer * buf)
     }
   }
 
-  return ret;
+  return GST_FLOW_OK;
 }
 
 /*** GSTURIHANDLER INTERFACE *************************************************/
 
-static GstURIType
+static guint
 gst_gnome_vfs_sink_uri_get_type (void)
 {
   return GST_URI_SINK;
@@ -586,7 +592,12 @@ gst_gnome_vfs_sink_uri_get_type (void)
 static gchar **
 gst_gnome_vfs_sink_uri_get_protocols (void)
 {
-  return gst_gnomevfs_get_supported_uris ();
+  static gchar **protocols = NULL;
+
+  if (!protocols)
+    protocols = gst_gnomevfs_get_supported_uris ();
+
+  return protocols;
 }
 
 static const gchar *
