@@ -27,6 +27,12 @@
 #include <string.h>
 #include <gst/gst-i18n-plugin.h>
 
+static const GstElementDetails gst_ogg_demux_details =
+GST_ELEMENT_DETAILS ("Ogg demuxer",
+    "Codec/Demuxer",
+    "demux ogg streams (info about ogg: http://xiph.org)",
+    "Wim Taymand <wim@fluendo.com>");
+
 #define CHUNKSIZE (8500)        /* this is out of vorbisfile */
 #define SKELETON_FISHEAD_SIZE 64
 #define SKELETON_FISBONE_MIN_SIZE 52
@@ -131,6 +137,8 @@ struct _GstOggPad
 
   ogg_stream_state stream;
 
+  gboolean discont;
+
   gboolean dynamic;             /* True if the internal element had dynamic pads */
   guint padaddedid;             /* The signal id for element::pad-added */
 };
@@ -161,6 +169,7 @@ struct _GstOggDemux
   GMutex *chain_lock;           /* we need the lock to protect the chains */
   GArray *chains;               /* list of chains we know */
   GstClockTime total_time;
+  GstFlowReturn chain_error;    /* error we received while finding chains */
 
   GstOggChain *current_chain;
   GstOggChain *building_chain;
@@ -259,7 +268,7 @@ gst_ogg_pad_class_init (GstOggPadClass * klass)
 
   gobject_class = (GObjectClass *) klass;
 
-  ogg_pad_parent_class = g_type_class_ref (GST_TYPE_PAD);
+  ogg_pad_parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->dispose = gst_ogg_pad_dispose;
   gobject_class->finalize = gst_ogg_pad_finalize;
@@ -293,13 +302,19 @@ static void
 gst_ogg_pad_dispose (GObject * object)
 {
   GstOggPad *pad = GST_OGG_PAD (object);
+  GstPad **elem_pad_p;
+  GstElement **element_p;
+  GstPad **elem_out_p;
 
   if (pad->element)
     gst_element_set_state (pad->element, GST_STATE_NULL);
 
-  gst_object_replace ((GstObject **) (&pad->elem_pad), NULL);
-  gst_object_replace ((GstObject **) (&pad->element), NULL);
-  gst_object_replace ((GstObject **) (&pad->elem_out), NULL);
+  elem_pad_p = &pad->elem_pad;
+  element_p = &pad->element;
+  elem_out_p = &pad->elem_out;
+  gst_object_replace ((GstObject **) elem_pad_p, NULL);
+  gst_object_replace ((GstObject **) element_p, NULL);
+  gst_object_replace ((GstObject **) elem_out_p, NULL);
 
   pad->chain = NULL;
   pad->ogg = NULL;
@@ -379,7 +394,7 @@ gst_ogg_pad_src_query (GstPad * pad, GstQuery * query)
   GstOggDemux *ogg;
   GstOggPad *cur;
 
-  ogg = GST_OGG_DEMUX (GST_PAD_PARENT (pad));
+  ogg = GST_OGG_DEMUX (gst_pad_get_parent (pad));
   cur = GST_OGG_PAD (pad);
 
   switch (GST_QUERY_TYPE (query)) {
@@ -389,12 +404,11 @@ gst_ogg_pad_src_query (GstPad * pad, GstQuery * query)
 
       gst_query_parse_duration (query, &format, NULL);
       /* can only get position in time */
-      if (format != GST_FORMAT_TIME) {
-        GST_DEBUG ("only query duration on TIME is supported");
-        res = FALSE;
-        goto done;
-      }
+      if (format != GST_FORMAT_TIME)
+        goto wrong_format;
+
       /* can only return the total time position */
+      /* FIXME, return time for this specific stream */
       gst_query_set_duration (query, GST_FORMAT_TIME, ogg->total_time);
       break;
     }
@@ -403,7 +417,17 @@ gst_ogg_pad_src_query (GstPad * pad, GstQuery * query)
       break;
   }
 done:
+  gst_object_unref (ogg);
+
   return res;
+
+  /* ERRORS */
+wrong_format:
+  {
+    GST_DEBUG_OBJECT (ogg, "only query duration on TIME is supported");
+    res = FALSE;
+    goto done;
+  }
 }
 
 static gboolean
@@ -416,32 +440,32 @@ gst_ogg_demux_receive_event (GstElement * element, GstEvent * event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
-    {
       /* can't seek if we are not seekable, FIXME could pass the
        * seek query upstream after converting it to bytes using
        * the average bitrate of the stream. */
       if (!ogg->seekable) {
-        GST_DEBUG ("seek on non seekable stream");
+        GST_DEBUG_OBJECT (ogg, "seek on non seekable stream");
         goto error;
       }
 
       /* now do the seek */
       res = gst_ogg_demux_perform_seek (ogg, event);
+      gst_event_unref (event);
       break;
-
-    }
     default:
-      GST_DEBUG ("We only handle seek events here");
+      GST_DEBUG_OBJECT (ogg, "We only handle seek events here");
       goto error;
-      break;
   }
 
   return res;
 
+  /* ERRORS */
 error:
-  GST_DEBUG ("error handling event");
-  gst_event_unref (event);
-  return FALSE;
+  {
+    GST_DEBUG_OBJECT (ogg, "error handling event");
+    gst_event_unref (event);
+    return FALSE;
+  }
 }
 
 static gboolean
@@ -451,36 +475,39 @@ gst_ogg_pad_event (GstPad * pad, GstEvent * event)
   GstOggDemux *ogg;
   GstOggPad *cur;
 
-  ogg = GST_OGG_DEMUX (GST_PAD_PARENT (pad));
+  ogg = GST_OGG_DEMUX (gst_pad_get_parent (pad));
   cur = GST_OGG_PAD (pad);
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
-    {
       /* can't seek if we are not seekable, FIXME could pass the
        * seek query upstream after converting it to bytes using
        * the average bitrate of the stream. */
       if (!ogg->seekable) {
-        GST_DEBUG ("seek on non seekable stream");
+        GST_DEBUG_OBJECT (ogg, "seek on non seekable stream");
         goto error;
       }
 
       /* now do the seek */
       res = gst_ogg_demux_perform_seek (ogg, event);
+      gst_event_unref (event);
       break;
-    }
     default:
       res = gst_pad_event_default (pad, event);
       break;
   }
+done:
+  gst_object_unref (ogg);
+
   return res;
 
   /* ERRORS */
 error:
   {
-    GST_DEBUG ("error handling event");
+    GST_DEBUG_OBJECT (ogg, "error handling event");
     gst_event_unref (event);
-    return FALSE;
+    res = FALSE;
+    goto done;
   }
 }
 
@@ -905,51 +932,83 @@ gst_ogg_demux_chain_peer (GstOggPad * pad, ogg_packet * packet)
   GstBuffer *buf;
   GstFlowReturn ret;
   GstOggDemux *ogg = pad->ogg;
-
-  ret =
-      gst_pad_alloc_buffer_and_set_caps (GST_PAD (pad), GST_BUFFER_OFFSET_NONE,
-      packet->bytes, GST_PAD_CAPS (pad), &buf);
+  GstFormat format;
+  gint64 current_time;
+  GstOggChain *chain;
 
   GST_DEBUG_OBJECT (ogg,
       "%p streaming to peer serial %08lx", pad, pad->serialno);
 
-  if (ret == GST_FLOW_OK) {
-    memcpy (buf->data, packet->packet, packet->bytes);
+  ret =
+      gst_pad_alloc_buffer_and_set_caps (GST_PAD (pad), GST_BUFFER_OFFSET_NONE,
+      packet->bytes, GST_PAD_CAPS (pad), &buf);
+  if (ret != GST_FLOW_OK)
+    goto no_buffer;
 
-    GST_BUFFER_OFFSET (buf) = -1;
-    GST_BUFFER_OFFSET_END (buf) = packet->granulepos;
+  /* copy packet in buffer */
+  memcpy (buf->data, packet->packet, packet->bytes);
 
-    ret = gst_pad_push (GST_PAD (pad), buf);
-    if (ret == GST_FLOW_NOT_LINKED)
-      ret = GST_FLOW_OK;
+  GST_BUFFER_OFFSET (buf) = -1;
+  GST_BUFFER_OFFSET_END (buf) = packet->granulepos;
 
-    if (packet->granulepos >= 0) {
-      GstFormat format;
-      gint64 current_time;
-
-      ogg->current_granule = packet->granulepos;
-      format = GST_FORMAT_TIME;
-      if (!pad->is_skeleton) {
-        if (!gst_ogg_pad_query_convert (pad,
-                GST_FORMAT_DEFAULT, packet->granulepos, &format,
-                (gint64 *) & current_time)) {
-          GST_WARNING_OBJECT (ogg, "could not convert granulepos to time");
-        } else {
-          gst_segment_set_last_stop (&ogg->segment, GST_FORMAT_TIME,
-              current_time);
-          GST_DEBUG ("ogg current time %" GST_TIME_FORMAT,
-              GST_TIME_ARGS (current_time));
-        }
-      }
-    }
-  } else {
-    GST_DEBUG_OBJECT (ogg,
-        "%p could not get buffer from peer %08lx", pad, pad->serialno);
-
-    if (ret == GST_FLOW_NOT_LINKED)
-      ret = GST_FLOW_OK;
+  /* Mark discont on the buffer */
+  if (pad->discont) {
+    GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
+    pad->discont = FALSE;
   }
+
+  ret = gst_pad_push (GST_PAD (pad), buf);
+  /* ignore not linked */
+  if (ret == GST_FLOW_NOT_LINKED)
+    ret = GST_FLOW_OK;
+
+  /* we're done with skeleton stuff */
+  if (pad->is_skeleton)
+    goto done;
+
+  /* check if valid granulepos, then we can calculate the current
+   * position */
+  if (packet->granulepos < 0)
+    goto done;
+
+  /* store current granule pos */
+  ogg->current_granule = packet->granulepos;
+
+  /* convert to time */
+  format = GST_FORMAT_TIME;
+  if (!gst_ogg_pad_query_convert (pad,
+          GST_FORMAT_DEFAULT, packet->granulepos, &format,
+          (gint64 *) & current_time))
+    goto convert_failed;
+
+  /* convert to stream time */
+  if ((chain = pad->chain))
+    current_time = current_time - chain->segment_start + chain->begin_time;
+
+  /* and store as the current position */
+  gst_segment_set_last_stop (&ogg->segment, GST_FORMAT_TIME, current_time);
+
+  GST_DEBUG_OBJECT (ogg, "ogg current time %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (current_time));
+
+done:
   return ret;
+
+  /* special cases */
+no_buffer:
+  {
+    GST_DEBUG_OBJECT (ogg,
+        "%p could not get buffer from peer %08lx, %d (%s)", pad,
+        pad->serialno, ret, gst_flow_get_name (ret));
+    if (ret == GST_FLOW_NOT_LINKED)
+      ret = GST_FLOW_OK;
+    return ret;
+  }
+convert_failed:
+  {
+    GST_WARNING_OBJECT (ogg, "could not convert granulepos to time");
+    return ret;
+  }
 }
 
 /* submit a packet to the oggpad, this function will run the
@@ -971,7 +1030,6 @@ gst_ogg_pad_submit_packet (GstOggPad * pad, ogg_packet * packet)
         !memcmp (packet->packet, "fishead\0", 8)) {
       gst_ogg_pad_parse_skeleton_fishead (pad, packet);
     }
-
     gst_ogg_pad_typefind (pad, packet);
     pad->have_type = TRUE;
   }
@@ -1007,16 +1065,17 @@ gst_ogg_pad_submit_packet (GstOggPad * pad, ogg_packet * packet)
       /* see if we have enough info to activate the chain */
       if (gst_ogg_demux_collect_chain_info (ogg, chain)) {
         GstEvent *event;
-        GstClockTime segment_start, segment_stop;
+        GstClockTime segment_start, segment_stop, segment_time;
 
-        GST_DEBUG ("chain->begin_time:    %" GST_TIME_FORMAT,
+        GST_DEBUG_OBJECT (ogg, "chain->begin_time:    %" GST_TIME_FORMAT,
             GST_TIME_ARGS (chain->begin_time));
-        GST_DEBUG ("chain->segment_start: %" GST_TIME_FORMAT,
+        GST_DEBUG_OBJECT (ogg, "chain->segment_start: %" GST_TIME_FORMAT,
             GST_TIME_ARGS (chain->segment_start));
-        GST_DEBUG ("chain->segment_stop:  %" GST_TIME_FORMAT,
+        GST_DEBUG_OBJECT (ogg, "chain->segment_stop:  %" GST_TIME_FORMAT,
             GST_TIME_ARGS (chain->segment_stop));
 
         if (chain->begin_time != GST_CLOCK_TIME_NONE) {
+          segment_time = chain->begin_time;
           segment_start = chain->segment_start - chain->begin_time;
           if (chain->segment_stop != GST_CLOCK_TIME_NONE) {
             segment_stop = chain->segment_stop - chain->begin_time;
@@ -1026,16 +1085,19 @@ gst_ogg_pad_submit_packet (GstOggPad * pad, ogg_packet * packet)
         } else {
           segment_start = chain->segment_start;
           segment_stop = chain->segment_stop;
+          segment_time = 0;
         }
 
-        GST_DEBUG ("segment_start: %" GST_TIME_FORMAT,
+        GST_DEBUG_OBJECT (ogg, "segment_start: %" GST_TIME_FORMAT,
             GST_TIME_ARGS (segment_start));
-        GST_DEBUG ("segment_stop:  %" GST_TIME_FORMAT,
+        GST_DEBUG_OBJECT (ogg, "segment_stop:  %" GST_TIME_FORMAT,
             GST_TIME_ARGS (segment_stop));
+        GST_DEBUG_OBJECT (ogg, "segment_time:  %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (segment_time));
 
         /* create the newsegment event we are going to send out */
         event = gst_event_new_new_segment (FALSE, ogg->segment.rate,
-            GST_FORMAT_TIME, segment_start, segment_stop, 0);
+            GST_FORMAT_TIME, segment_start, segment_stop, segment_time);
 
         gst_ogg_demux_activate_chain (ogg, chain, event);
 
@@ -1147,7 +1209,19 @@ gst_ogg_chain_free (GstOggChain * chain)
     gst_object_unref (pad);
   }
   g_array_free (chain->streams, TRUE);
-  chain->streams = NULL;
+  g_free (chain);
+}
+
+static void
+gst_ogg_chain_mark_discont (GstOggChain * chain)
+{
+  gint i;
+
+  for (i = 0; i < chain->streams->len; i++) {
+    GstOggPad *pad = g_array_index (chain->streams, GstOggPad *, i);
+
+    pad->discont = TRUE;
+  }
 }
 
 static GstOggPad *
@@ -1171,6 +1245,7 @@ gst_ogg_chain_new_stream (GstOggChain * chain, glong serialno)
   GST_PAD_DIRECTION (ret) = GST_PAD_SRC;
   ret->chain = chain;
   ret->ogg = chain->ogg;
+  ret->discont = TRUE;
   gst_object_set_name (GST_OBJECT (ret), name);
   g_free (name);
 
@@ -1251,7 +1326,7 @@ static GstOggChain *gst_ogg_demux_read_chain (GstOggDemux * ogg);
 static gint gst_ogg_demux_read_end_chain (GstOggDemux * ogg,
     GstOggChain * chain);
 
-static gboolean gst_ogg_demux_handle_event (GstPad * pad, GstEvent * event);
+static gboolean gst_ogg_demux_sink_event (GstPad * pad, GstEvent * event);
 static void gst_ogg_demux_loop (GstOggPad * pad);
 static GstFlowReturn gst_ogg_demux_chain (GstPad * pad, GstBuffer * buffer);
 static gboolean gst_ogg_demux_sink_activate (GstPad * sinkpad);
@@ -1271,11 +1346,6 @@ static void
 gst_ogg_demux_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-  static GstElementDetails gst_ogg_demux_details =
-      GST_ELEMENT_DETAILS ("ogg demuxer",
-      "Codec/Demuxer",
-      "demux ogg streams (info about ogg: http://xiph.org)",
-      "Wim Taymand <wim@fluendo.com>");
 
   gst_element_class_set_details (element_class, &gst_ogg_demux_details);
 
@@ -1304,7 +1374,7 @@ gst_ogg_demux_init (GstOggDemux * ogg, GstOggDemuxClass * g_class)
       gst_pad_new_from_static_template (&ogg_demux_sink_template_factory,
       "sink");
 
-  gst_pad_set_event_function (ogg->sinkpad, gst_ogg_demux_handle_event);
+  gst_pad_set_event_function (ogg->sinkpad, gst_ogg_demux_sink_event);
   gst_pad_set_chain_function (ogg->sinkpad, gst_ogg_demux_chain);
   gst_pad_set_activate_function (ogg->sinkpad, gst_ogg_demux_sink_activate);
   gst_pad_set_activatepull_function (ogg->sinkpad,
@@ -1328,25 +1398,33 @@ gst_ogg_demux_finalize (GObject * object)
   g_mutex_free (ogg->chain_lock);
   ogg_sync_clear (&ogg->sync);
 
-  if (G_OBJECT_CLASS (parent_class)->finalize)
-    G_OBJECT_CLASS (parent_class)->finalize (object);
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static gboolean
-gst_ogg_demux_handle_event (GstPad * pad, GstEvent * event)
+gst_ogg_demux_sink_event (GstPad * pad, GstEvent * event)
 {
-  GstOggDemux *ogg = GST_OGG_DEMUX (GST_PAD_PARENT (pad));
+  gboolean res;
+  GstOggDemux *ogg;
+
+  ogg = GST_OGG_DEMUX (gst_pad_get_parent (pad));
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_NEWSEGMENT:
+      /* FIXME */
       GST_DEBUG_OBJECT (ogg, "got a new segment event");
       ogg_sync_reset (&ogg->sync);
       gst_event_unref (event);
+      res = TRUE;
       break;
+    case GST_EVENT_EOS:
     default:
-      return gst_pad_event_default (pad, event);
+      res = gst_pad_event_default (pad, event);
+      break;
   }
-  return TRUE;
+  gst_object_unref (ogg);
+
+  return res;
 }
 
 /* submit the given buffer to the ogg sync.
@@ -1356,7 +1434,7 @@ gst_ogg_demux_handle_event (GstPad * pad, GstEvent * event)
 static gint
 gst_ogg_demux_submit_buffer (GstOggDemux * ogg, GstBuffer * buffer)
 {
-  guint size;
+  gint size;
   guint8 *data;
   gchar *oggbuffer;
 
@@ -1387,7 +1465,7 @@ gst_ogg_demux_seek (GstOggDemux * ogg, gint64 offset)
 /* read more data from the current offset and submit to
  * the ogg sync layer.
  *
- * Return number of bytes written.
+ * Return number of bytes written or 0 on EOS or -1 on error.
  */
 static gint
 gst_ogg_demux_get_data (GstOggDemux * ogg)
@@ -1398,15 +1476,29 @@ gst_ogg_demux_get_data (GstOggDemux * ogg)
 
   GST_LOG_OBJECT (ogg, "get data %lld %lld", ogg->offset, ogg->length);
   if (ogg->offset == ogg->length)
-    return 0;
+    goto eos;
 
   ret = gst_pad_pull_range (ogg->sinkpad, ogg->offset, CHUNKSIZE, &buffer);
   if (ret != GST_FLOW_OK)
-    return -1;
+    goto error;
 
   size = gst_ogg_demux_submit_buffer (ogg, buffer);
 
   return size;
+
+  /* ERROR */
+eos:
+  {
+    GST_LOG_OBJECT (ogg, "reached EOS");
+    return 0;
+  }
+error:
+  {
+    GST_WARNING_OBJECT (ogg, "got %d (%s) from pull range", ret,
+        gst_flow_get_name (ret));
+    ogg->chain_error = ret;
+    return -1;
+  }
 }
 
 /* Read the next page from the current offset.
@@ -1518,7 +1610,7 @@ gst_ogg_demux_deactivate_current_chain (GstOggDemux * ogg)
   if (chain == NULL)
     return TRUE;
 
-  GST_DEBUG ("deactivating chain %p", chain);
+  GST_DEBUG_OBJECT (ogg, "deactivating chain %p", chain);
 
   /* send EOS on all the pads */
   for (i = 0; i < chain->streams->len; i++) {
@@ -1563,7 +1655,7 @@ gst_ogg_demux_activate_chain (GstOggDemux * ogg, GstOggChain * chain,
     return TRUE;
   }
 
-  GST_DEBUG ("activating chain %p", chain);
+  GST_DEBUG_OBJECT (ogg, "activating chain %p", chain);
 
   /* first add the pads */
   for (i = 0; i < chain->streams->len; i++) {
@@ -1572,6 +1664,9 @@ gst_ogg_demux_activate_chain (GstOggDemux * ogg, GstOggChain * chain,
     pad = g_array_index (chain->streams, GstOggPad *, i);
     GST_DEBUG_OBJECT (ogg, "adding pad %" GST_PTR_FORMAT, pad);
 
+    /* mark discont */
+    pad->discont = TRUE;
+
     /* activate first */
     gst_pad_set_active (GST_PAD_CAST (pad), TRUE);
 
@@ -1584,7 +1679,7 @@ gst_ogg_demux_activate_chain (GstOggDemux * ogg, GstOggChain * chain,
   if (event)
     gst_ogg_demux_send_event (ogg, event);
 
-  GST_DEBUG ("starting chain");
+  GST_DEBUG_OBJECT (ogg, "starting chain");
 
   /* then send out any queued buffers */
   for (i = 0; i < chain->streams->len; i++) {
@@ -1596,6 +1691,11 @@ gst_ogg_demux_activate_chain (GstOggDemux * ogg, GstOggChain * chain,
     for (headers = pad->headers; headers; headers = g_list_next (headers)) {
       GstBuffer *buffer = GST_BUFFER (headers->data);
 
+      if (pad->discont) {
+        GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DISCONT);
+        pad->discont = FALSE;
+      }
+
       /* we don't care about the return value here */
       gst_pad_push (GST_PAD_CAST (pad), buffer);
     }
@@ -1778,6 +1878,7 @@ seek_error:
   }
 }
 
+/* does not take ownership of the event */
 static gboolean
 gst_ogg_demux_perform_seek (GstOggDemux * ogg, GstEvent * event)
 {
@@ -1792,6 +1893,8 @@ gst_ogg_demux_perform_seek (GstOggDemux * ogg, GstEvent * event)
   gboolean update;
 
   if (event) {
+    GST_DEBUG_OBJECT (ogg, "seek with event");
+
     gst_event_parse_seek (event, &rate, &format, &flags,
         &cur_type, &cur, &stop_type, &stop);
 
@@ -1806,7 +1909,10 @@ gst_ogg_demux_perform_seek (GstOggDemux * ogg, GstEvent * event)
       goto error;
     }
   } else {
+    GST_DEBUG_OBJECT (ogg, "seek without event");
+
     flags = 0;
+    rate = 1.0;
   }
 
   GST_DEBUG_OBJECT (ogg, "seek, rate %g", rate);
@@ -1845,15 +1951,17 @@ gst_ogg_demux_perform_seek (GstOggDemux * ogg, GstEvent * event)
   GST_PAD_STREAM_LOCK (ogg->sinkpad);
 
   if (ogg->segment_running && !flush) {
-    GstEvent *newseg;
+    /* create the segment event to close the current segment */
+    if ((chain = ogg->current_chain)) {
+      GstEvent *newseg;
 
-    /* create the discont event to close the current segment */
-    newseg = gst_event_new_new_segment (TRUE, ogg->segment.rate,
-        GST_FORMAT_TIME, ogg->segment.start, ogg->segment.last_stop,
-        ogg->segment.start);
+      newseg = gst_event_new_new_segment (TRUE, ogg->segment.rate,
+          GST_FORMAT_TIME, ogg->segment.start + chain->segment_start,
+          ogg->segment.last_stop + chain->segment_start, ogg->segment.time);
 
-    /* send discont on old chain */
-    gst_ogg_demux_send_event (ogg, newseg);
+      /* send segment on old chain */
+      gst_ogg_demux_send_event (ogg, newseg);
+    }
   }
 
   if (event) {
@@ -1861,7 +1969,7 @@ gst_ogg_demux_perform_seek (GstOggDemux * ogg, GstEvent * event)
         cur_type, cur, stop_type, stop, &update);
   }
 
-  GST_DEBUG ("segment positions set to %" GST_TIME_FORMAT "-%"
+  GST_DEBUG_OBJECT (ogg, "segment positions set to %" GST_TIME_FORMAT "-%"
       GST_TIME_FORMAT, GST_TIME_ARGS (ogg->segment.start),
       GST_TIME_ARGS (ogg->segment.stop));
 
@@ -1900,15 +2008,20 @@ gst_ogg_demux_perform_seek (GstOggDemux * ogg, GstEvent * event)
     if ((stop = ogg->segment.stop) == -1)
       stop = ogg->segment.duration;
 
-    /* create the discont event we are going to send out */
+    if (stop != -1)
+      stop += chain->segment_start;
+
+    /* create the segment event we are going to send out */
     event = gst_event_new_new_segment (FALSE, ogg->segment.rate,
-        ogg->segment.format, ogg->segment.last_stop, stop, ogg->segment.time);
+        ogg->segment.format,
+        ogg->segment.last_stop + chain->segment_start, stop, ogg->segment.time);
 
     if (chain != ogg->current_chain) {
-      /* switch to different chain, send discont on new chain */
+      /* switch to different chain, send segment on new chain */
       gst_ogg_demux_activate_chain (ogg, chain, event);
     } else {
-      /* send discont on old chain */
+      /* mark discont and send segment on current chain */
+      gst_ogg_chain_mark_discont (chain);
       gst_ogg_demux_send_event (ogg, event);
     }
 
@@ -1971,7 +2084,7 @@ gst_ogg_demux_bisect_forward_serialno (GstOggDemux * ogg,
     gst_ogg_demux_seek (ogg, bisect);
     ret = gst_ogg_demux_get_next_page (ogg, &og, -1);
     if (ret == OV_EREAD) {
-      GST_LOG_OBJECT (ogg, "OV_READ");
+      GST_LOG_OBJECT (ogg, "OV_EREAD");
       return OV_EREAD;
     }
 
@@ -2328,7 +2441,7 @@ gst_ogg_demux_find_chains (GstOggDemux * ogg)
   if (!res || ogg->length <= 0)
     goto no_length;
 
-  GST_DEBUG ("file length %lld", ogg->length);
+  GST_DEBUG_OBJECT (ogg, "file length %lld", ogg->length);
 
   /* read chain from offset 0, this is the first chain of the
    * ogg file. */
@@ -2364,17 +2477,17 @@ gst_ogg_demux_find_chains (GstOggDemux * ogg)
   /*** error cases ***/
 no_peer:
   {
-    GST_DEBUG ("we don't have a peer");
+    GST_DEBUG_OBJECT (ogg, "we don't have a peer");
     return FALSE;
   }
 no_length:
   {
-    GST_DEBUG ("can't get file length");
+    GST_DEBUG_OBJECT (ogg, "can't get file length");
     return FALSE;
   }
 no_first_chain:
   {
-    GST_DEBUG ("can't get first chain");
+    GST_DEBUG_OBJECT (ogg, "can't get first chain");
     return FALSE;
   }
 }
@@ -2392,7 +2505,7 @@ gst_ogg_demux_chain (GstPad * pad, GstBuffer * buffer)
 
   ogg = GST_OGG_DEMUX (GST_OBJECT_PARENT (pad));
 
-  GST_DEBUG ("chain");
+  GST_DEBUG_OBJECT (ogg, "chain");
   gst_ogg_demux_submit_buffer (ogg, buffer);
 
   while (ret != 0 && result == GST_FLOW_OK) {
@@ -2458,7 +2571,7 @@ gst_ogg_demux_chain (GstPad * pad, GstBuffer * buffer)
             newchain->offset = 0;
             /* set new chain begin time aligned with end time of old chain */
             newchain->begin_time = chain_time;
-            GST_DEBUG ("new chain, begin time %" GST_TIME_FORMAT,
+            GST_DEBUG_OBJECT (ogg, "new chain, begin time %" GST_TIME_FORMAT,
                 GST_TIME_ARGS (chain_time));
 
             /* and this is the one we are building now */
@@ -2540,6 +2653,7 @@ gst_ogg_demux_loop (GstOggPad * pad)
 
     /* this is the only place where we write chains */
     GST_CHAIN_LOCK (ogg);
+    ogg->chain_error = GST_FLOW_OK;
     got_chains = gst_ogg_demux_find_chains (ogg);
     GST_CHAIN_UNLOCK (ogg);
     if (!got_chains)
@@ -2555,27 +2669,13 @@ gst_ogg_demux_loop (GstOggPad * pad)
 
     /* and seek to configured positions without FLUSH */
     gst_ogg_demux_perform_seek (ogg, event);
+    if (event)
+      gst_event_unref (event);
   }
 
   GST_LOG_OBJECT (ogg, "pull data %lld", ogg->offset);
-  if (ogg->offset == ogg->length) {
-    ret = GST_FLOW_OK;
-    /* segment playback just posts a segment end message instead of
-     * pushing out EOS. */
-    /* FIXME, need to be done somewhere else where we
-     * can check against segment_stop time. */
-    ogg->segment_running = FALSE;
-    if (ogg->segment.flags & GST_SEEK_FLAG_SEGMENT) {
-      GST_LOG_OBJECT (ogg, "Sending segment done, at end of segment");
-      gst_element_post_message (GST_ELEMENT (ogg),
-          gst_message_new_segment_done (GST_OBJECT (ogg), GST_FORMAT_TIME,
-              ogg->total_time));
-    } else {
-      GST_LOG_OBJECT (ogg, "Sending EOS, at end of stream");
-      gst_ogg_demux_send_event (ogg, gst_event_new_eos ());
-    }
-    goto pause;
-  }
+  if (ogg->offset == ogg->length)
+    goto eos;
 
   ret = gst_pad_pull_range (ogg->sinkpad, ogg->offset, CHUNKSIZE, &buffer);
   if (ret != GST_FLOW_OK) {
@@ -2591,13 +2691,40 @@ gst_ogg_demux_loop (GstOggPad * pad)
     goto pause;
   }
 
+  /* check for the end of the segment */
+  if (ogg->segment.stop != -1 && ogg->segment.last_stop != -1)
+    if (ogg->segment.last_stop > ogg->segment.stop)
+      goto eos;
+
   return;
 
   /* ERRORS */
 chain_read_failed:
   {
     GST_ELEMENT_ERROR (ogg, STREAM, DEMUX, (NULL), ("could not read chains"));
-    ret = GST_FLOW_ERROR;
+    ret = ogg->chain_error;
+    goto pause;
+  }
+eos:
+  {
+    ret = GST_FLOW_OK;
+    /* segment playback just posts a segment end message instead of
+     * pushing out EOS. */
+    ogg->segment_running = FALSE;
+    if (ogg->segment.flags & GST_SEEK_FLAG_SEGMENT) {
+      gint64 stop;
+
+      if ((stop = ogg->segment.stop) == -1)
+        stop = ogg->segment.duration;
+
+      GST_LOG_OBJECT (ogg, "Sending segment done, at end of segment");
+      gst_element_post_message (GST_ELEMENT (ogg),
+          gst_message_new_segment_done (GST_OBJECT (ogg), GST_FORMAT_TIME,
+              stop));
+    } else {
+      GST_LOG_OBJECT (ogg, "Sending EOS, at end of stream");
+      gst_ogg_demux_send_event (ogg, gst_event_new_eos ());
+    }
     goto pause;
   }
 pause:
