@@ -55,6 +55,24 @@ GST_DEBUG_CATEGORY_STATIC (gst_ogm_parse_debug);
 #define GST_OGM_PARSE_GET_CLASS(obj) \
   (G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_OGM_PARSE, GstOgmParseClass))
 
+static const GstElementDetails gst_ogm_audio_parse_details =
+GST_ELEMENT_DETAILS ("OGM audio stream parser",
+    "Codec/Decoder/Audio",
+    "parse an OGM audio header and stream",
+    "Ronald Bultje <rbultje@ronald.bitfreak.net>");
+
+static const GstElementDetails gst_ogm_video_parse_details =
+GST_ELEMENT_DETAILS ("OGM video stream parser",
+    "Codec/Decoder/Video",
+    "parse an OGM video header and stream",
+    "Ronald Bultje <rbultje@ronald.bitfreak.net>");
+
+static const GstElementDetails gst_ogm_text_parse_details =
+GST_ELEMENT_DETAILS ("OGM text stream parser",
+    "Codec/Decoder/Subtitle",
+    "parse an OGM text header and stream",
+    "Ronald Bultje <rbultje@ronald.bitfreak.net>");
+
 typedef struct _stream_header_video
 {
   gint32 width;
@@ -140,10 +158,6 @@ static void gst_ogm_video_parse_init (GstOgmParse * ogm);
 static void gst_ogm_audio_parse_init (GstOgmParse * ogm);
 static void gst_ogm_text_parse_init (GstOgmParse * ogm);
 
-#if 0
-static const GstFormat *gst_ogm_parse_get_sink_formats (GstPad * pad);
-#endif
-
 static const GstQueryType *gst_ogm_parse_get_sink_querytypes (GstPad * pad);
 static gboolean gst_ogm_parse_sink_query (GstPad * pad, GstQuery * query);
 static gboolean gst_ogm_parse_sink_convert (GstPad * pad, GstFormat src_format,
@@ -264,11 +278,6 @@ static void
 gst_ogm_audio_parse_base_init (GstOgmParseClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
-  static GstElementDetails gst_ogm_audio_parse_details =
-      GST_ELEMENT_DETAILS ("OGM audio stream parser",
-      "Codec/Decoder/Audio",
-      "parse an OGM audio header and stream",
-      "Ronald Bultje <rbultje@ronald.bitfreak.net>");
   GstCaps *caps = gst_riff_create_audio_template_caps ();
 
   gst_element_class_set_details (element_class, &gst_ogm_audio_parse_details);
@@ -284,11 +293,6 @@ static void
 gst_ogm_video_parse_base_init (GstOgmParseClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
-  static GstElementDetails gst_ogm_video_parse_details =
-      GST_ELEMENT_DETAILS ("OGM video stream parser",
-      "Codec/Decoder/Video",
-      "parse an OGM video header and stream",
-      "Ronald Bultje <rbultje@ronald.bitfreak.net>");
   GstCaps *caps = gst_riff_create_video_template_caps ();
 
   gst_element_class_set_details (element_class, &gst_ogm_video_parse_details);
@@ -304,11 +308,6 @@ static void
 gst_ogm_text_parse_base_init (GstOgmParseClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
-  static GstElementDetails gst_ogm_text_parse_details =
-      GST_ELEMENT_DETAILS ("OGM text stream parser",
-      "Codec/Decoder/Subtitle",
-      "parse an OGM text header and stream",
-      "Ronald Bultje <rbultje@ronald.bitfreak.net>");
   GstCaps *caps = gst_caps_new_simple ("text/plain", NULL, NULL);
 
   gst_element_class_set_details (element_class, &gst_ogm_text_parse_details);
@@ -325,9 +324,10 @@ gst_ogm_parse_class_init (GstOgmParseClass * klass)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
 
-  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
+  parent_class = g_type_class_peek_parent (klass);
 
-  gstelement_class->change_state = gst_ogm_parse_change_state;
+  gstelement_class->change_state =
+      GST_DEBUG_FUNCPTR (gst_ogm_parse_change_state);
 }
 
 static void
@@ -346,15 +346,12 @@ gst_ogm_audio_parse_init (GstOgmParse * ogm)
   ogm->sinkpad =
       gst_pad_new_from_static_template (&ogm_audio_parse_sink_template_factory,
       "sink");
-  gst_pad_set_query_function (ogm->sinkpad, gst_ogm_parse_sink_query);
-  gst_pad_set_chain_function (ogm->sinkpad, gst_ogm_parse_chain);
+  gst_pad_set_query_function (ogm->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_ogm_parse_sink_query));
+  gst_pad_set_chain_function (ogm->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_ogm_parse_chain));
   gst_element_add_pad (GST_ELEMENT (ogm), ogm->sinkpad);
 
-#if 0
-  ogm->srcpad = gst_pad_new_from_template (audio_src_templ, "src");
-  gst_pad_use_explicit_caps (ogm->srcpad);
-  gst_element_add_pad (GST_ELEMENT (ogm), ogm->srcpad);
-#endif
   ogm->srcpad = NULL;
   ogm->srcpadtempl = audio_src_templ;
 }
@@ -366,15 +363,12 @@ gst_ogm_video_parse_init (GstOgmParse * ogm)
   ogm->sinkpad =
       gst_pad_new_from_static_template (&ogm_video_parse_sink_template_factory,
       "sink");
-  gst_pad_set_query_function (ogm->sinkpad, gst_ogm_parse_sink_query);
-  gst_pad_set_chain_function (ogm->sinkpad, gst_ogm_parse_chain);
+  gst_pad_set_query_function (ogm->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_ogm_parse_sink_query));
+  gst_pad_set_chain_function (ogm->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_ogm_parse_chain));
   gst_element_add_pad (GST_ELEMENT (ogm), ogm->sinkpad);
 
-#if 0
-  ogm->srcpad = gst_pad_new_from_template (video_src_templ, "src");
-  gst_pad_use_explicit_caps (ogm->srcpad);
-  gst_element_add_pad (GST_ELEMENT (ogm), ogm->srcpad);
-#endif
   ogm->srcpad = NULL;
   ogm->srcpadtempl = video_src_templ;
 }
@@ -388,33 +382,16 @@ gst_ogm_text_parse_init (GstOgmParse * ogm)
       "sink");
   gst_pad_set_query_type_function (ogm->sinkpad,
       gst_ogm_parse_get_sink_querytypes);
-  gst_pad_set_query_function (ogm->sinkpad, gst_ogm_parse_sink_query);
-  gst_pad_set_chain_function (ogm->sinkpad, gst_ogm_parse_chain);
+  gst_pad_set_query_function (ogm->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_ogm_parse_sink_query));
+  gst_pad_set_chain_function (ogm->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_ogm_parse_chain));
   gst_element_add_pad (GST_ELEMENT (ogm), ogm->sinkpad);
 
-#if 0
-  ogm->srcpad = gst_pad_new_from_template (text_src_templ, "src");
-  gst_pad_use_explicit_caps (ogm->srcpad);
-  gst_element_add_pad (GST_ELEMENT (ogm), ogm->srcpad);
-#endif
   ogm->srcpad = NULL;
   ogm->srcpadtempl = text_src_templ;
 }
 
-#if 0
-static const GstFormat *
-gst_ogm_parse_get_sink_formats (GstPad * pad)
-{
-  static GstFormat formats[] = {
-    GST_FORMAT_DEFAULT,
-    GST_FORMAT_TIME,
-    0
-  };
-
-  return formats;
-}
-#endif
-
 static const GstQueryType *
 gst_ogm_parse_get_sink_querytypes (GstPad * pad)
 {
@@ -483,15 +460,16 @@ gst_ogm_parse_sink_convert (GstPad * pad,
       break;
   }
 
+  gst_object_unref (ogm);
   return res;
 }
 
 static gboolean
 gst_ogm_parse_sink_query (GstPad * pad, GstQuery * query)
 {
-  GstOgmParse *ogm = GST_OGM_PARSE (GST_PAD_PARENT (pad));
+  GstOgmParse *ogm = GST_OGM_PARSE (gst_pad_get_parent (pad));
   GstFormat format;
-  gboolean res;
+  gboolean res = FALSE;
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_POSITION:
@@ -501,7 +479,7 @@ gst_ogm_parse_sink_query (GstPad * pad, GstQuery * query)
       gst_query_parse_position (query, &format, NULL);
 
       if (format != GST_FORMAT_DEFAULT && format != GST_FORMAT_TIME)
-        return FALSE;
+        break;
 
       if ((res = gst_ogm_parse_sink_convert (pad,
                   GST_FORMAT_DEFAULT, ogm->next_granulepos, &format, &val))) {
@@ -527,13 +505,16 @@ gst_ogm_parse_sink_query (GstPad * pad, GstQuery * query)
       res = gst_pad_query_default (pad, query);
       break;
   }
+
+  gst_object_unref (ogm);
   return res;
 }
 
 static GstFlowReturn
 gst_ogm_parse_chain (GstPad * pad, GstBuffer * buffer)
 {
-  GstOgmParse *ogm = GST_OGM_PARSE (gst_pad_get_parent (pad));
+  GstFlowReturn ret = GST_FLOW_OK;
+  GstOgmParse *ogm = GST_OGM_PARSE (GST_PAD_PARENT (pad));
   GstBuffer *buf = GST_BUFFER (buffer);
   guint8 *data = GST_BUFFER_DATA (buf);
   guint size = GST_BUFFER_SIZE (buf);
@@ -628,21 +609,28 @@ gst_ogm_parse_chain (GstPad * pad, GstBuffer * buffer)
       }
 
       if (ogm->srcpad) {
-        GST_WARNING_OBJECT (ogm, "Already an existing pad %s:%s",
-            GST_DEBUG_PAD_NAME (ogm->srcpad));
-        gst_element_remove_pad (GST_ELEMENT (ogm), ogm->srcpad);
-        gst_object_unref (ogm->srcpad);
+        GstCaps *current_caps = GST_PAD_CAPS (ogm->srcpad);
 
+        if (current_caps && !gst_caps_is_equal (current_caps, caps)) {
+          GST_WARNING_OBJECT (ogm, "Already an existing pad %s:%s",
+              GST_DEBUG_PAD_NAME (ogm->srcpad));
+          gst_element_remove_pad (GST_ELEMENT (ogm), ogm->srcpad);
+          ogm->srcpad = NULL;
+        } else {
+          GST_DEBUG_OBJECT (ogm, "Existing pad has the same caps, do nothing");
+        }
       }
-      if (caps) {
-        ogm->srcpad = gst_pad_new ("src", GST_PAD_SRC);
-        gst_pad_set_caps (ogm->srcpad, caps);
-      } else {
-        GST_WARNING_OBJECT (ogm,
-            "No fixed caps were found, carrying on with template");
-        ogm->srcpad = gst_pad_new_from_template (ogm->srcpadtempl, "src");
+      if (ogm->srcpad == NULL) {
+        if (caps) {
+          ogm->srcpad = gst_pad_new ("src", GST_PAD_SRC);
+          gst_pad_set_caps (ogm->srcpad, caps);
+        } else {
+          GST_WARNING_OBJECT (ogm,
+              "No fixed caps were found, carrying on with template");
+          ogm->srcpad = gst_pad_new_from_template (ogm->srcpadtempl, "src");
+        }
+        gst_element_add_pad (GST_ELEMENT (ogm), ogm->srcpad);
       }
-      gst_element_add_pad (GST_ELEMENT (ogm), ogm->srcpad);
       break;
     }
     case 0x03:
@@ -701,7 +689,10 @@ gst_ogm_parse_chain (GstPad * pad, GstBuffer * buffer)
             break;
         }
         gst_buffer_set_caps (sbuf, GST_PAD_CAPS (ogm->srcpad));
-        gst_pad_push (ogm->srcpad, sbuf);
+        ret = gst_pad_push (ogm->srcpad, sbuf);
+        if (ret != GST_FLOW_OK) {
+          GST_DEBUG_OBJECT (ogm, "Flow return: %s", gst_flow_get_name (ret));
+        }
       } else {
         GST_ELEMENT_ERROR (ogm, STREAM, WRONG_TYPE,
             ("Wrong packet startcode 0x%02x", data[0]), (NULL));
@@ -711,14 +702,19 @@ gst_ogm_parse_chain (GstPad * pad, GstBuffer * buffer)
 
   gst_buffer_unref (buf);
 
-  return GST_FLOW_OK;
+  return ret;
 }
 
 static GstStateChangeReturn
 gst_ogm_parse_change_state (GstElement * element, GstStateChange transition)
 {
+  GstStateChangeReturn ret;
   GstOgmParse *ogm = GST_OGM_PARSE (element);
 
+  ret = parent_class->change_state (element, transition);
+  if (ret != GST_STATE_CHANGE_SUCCESS)
+    return ret;
+
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
       if (ogm->srcpad) {
@@ -732,7 +728,7 @@ gst_ogm_parse_change_state (GstElement * element, GstStateChange transition)
       break;
   }
 
-  return parent_class->change_state (element, transition);
+  return ret;
 }
 
 gboolean
