@@ -44,7 +44,7 @@
  * </programlisting>
  * </para>
  * <para>
- * Here is another pipeline that displays subtitles from an .srt subtitle 
+ * Here is another pipeline that displays subtitles from an .srt subtitle
  * file, centered at the bottom of the picture and with a rectangular shading
  * around the text in the background:
  * <programlisting>
@@ -56,12 +56,12 @@
  * 1
  * 00:00:03,000 --> 00:00:05,000
  * Hello? (3-5s)
- * 
+ *  
  * 2
  * 00:00:08,000 --> 00:00:13,000
  * Yes, this is a subtitle. Don&apos;t
  * you like it? (8-13s)
- * 
+ *  
  * 3
  * 00:00:18,826 --> 00:01:02,886
  * Uh? What are you talking about?
@@ -93,12 +93,11 @@
 GST_DEBUG_CATEGORY (pango_debug);
 #define GST_CAT_DEFAULT pango_debug
 
-static GstElementDetails text_overlay_details = {
-  "Text Overlay",
-  "Filter/Editor/Video",
-  "Adds text strings on top of a video buffer",
-  "David Schleef <ds@schleef.org>"
-};
+static const GstElementDetails text_overlay_details =
+GST_ELEMENT_DETAILS ("Text overlay",
+    "Filter/Editor/Video",
+    "Adds text strings on top of a video buffer",
+    "David Schleef <ds@schleef.org>");
 
 
 #define DEFAULT_PROP_TEXT 	""
@@ -162,7 +161,7 @@ static GType
 gst_text_overlay_valign_get_type (void)
 {
   static GType text_overlay_valign_type = 0;
-  static GEnumValue text_overlay_valign[] = {
+  static const GEnumValue text_overlay_valign[] = {
     {GST_TEXT_OVERLAY_VALIGN_BASELINE, "baseline", "baseline"},
     {GST_TEXT_OVERLAY_VALIGN_BOTTOM, "bottom", "bottom"},
     {GST_TEXT_OVERLAY_VALIGN_TOP, "top", "top"},
@@ -181,7 +180,7 @@ static GType
 gst_text_overlay_halign_get_type (void)
 {
   static GType text_overlay_halign_type = 0;
-  static GEnumValue text_overlay_halign[] = {
+  static const GEnumValue text_overlay_halign[] = {
     {GST_TEXT_OVERLAY_HALIGN_LEFT, "left", "left"},
     {GST_TEXT_OVERLAY_HALIGN_CENTER, "center", "center"},
     {GST_TEXT_OVERLAY_HALIGN_RIGHT, "right", "right"},
@@ -201,7 +200,7 @@ static GType
 gst_text_overlay_wrap_mode_get_type (void)
 {
   static GType text_overlay_wrap_mode_type = 0;
-  static GEnumValue text_overlay_wrap_mode[] = {
+  static const GEnumValue text_overlay_wrap_mode[] = {
     {GST_TEXT_OVERLAY_WRAP_MODE_NONE, "none", "none"},
     {GST_TEXT_OVERLAY_WRAP_MODE_WORD, "word", "word"},
     {GST_TEXT_OVERLAY_WRAP_MODE_CHAR, "char", "char"},
@@ -437,6 +436,7 @@ gst_text_overlay_init (GstTextOverlay * overlay, GstTextOverlayClass * klass)
 
   overlay->layout =
       pango_layout_new (GST_TEXT_OVERLAY_GET_CLASS (overlay)->pango_context);
+  pango_layout_set_alignment (overlay->layout, PANGO_ALIGN_CENTER);
   memset (&overlay->bitmap, 0, sizeof (overlay->bitmap));
 
   overlay->halign = DEFAULT_PROP_HALIGNMENT;
@@ -951,7 +951,7 @@ gst_text_overlay_render_text (GstTextOverlay * overlay,
   } else {                      /* empty string */
     string = g_strdup (" ");
   }
-  g_strdelimit (string, "\n\r\t", ' ');
+  g_strdelimit (string, "\r\t", ' ');
   textlen = strlen (string);
 
   /* FIXME: should we check for UTF-8 here? */
