@@ -2,7 +2,6 @@
 #define __GST_TEXT_OVERLAY_H__
 
 #include <gst/gst.h>
-#include <gst/base/gstcollectpads.h>
 #include <pango/pangoft2.h>
 
 G_BEGIN_DECLS
@@ -22,30 +21,55 @@ G_BEGIN_DECLS
 typedef struct _GstTextOverlay      GstTextOverlay;
 typedef struct _GstTextOverlayClass GstTextOverlayClass;
 
-typedef enum _GstTextOverlayVAlign   GstTextOverlayVAlign;
-typedef enum _GstTextOverlayHAlign   GstTextOverlayHAlign;
-typedef enum _GstTextOverlayWrapMode GstTextOverlayWrapMode;
-
-enum _GstTextOverlayVAlign {
+/**
+ * GstTextOverlayVAlign:
+ * @GST_TEXT_OVERLAY_VALIGN_BASELINE: draw text on the baseline
+ * @GST_TEXT_OVERLAY_VALIGN_BOTTOM: draw text on the bottom
+ * @GST_TEXT_OVERLAY_VALIGN_TOP: draw test on top
+ *
+ * Vertical alignment of the text.
+ */
+typedef enum {
     GST_TEXT_OVERLAY_VALIGN_BASELINE,
     GST_TEXT_OVERLAY_VALIGN_BOTTOM,
     GST_TEXT_OVERLAY_VALIGN_TOP
-};
+} GstTextOverlayVAlign;
 
-enum _GstTextOverlayHAlign {
+/**
+ * GstTextOverlayHAlign:
+ * @GST_TEXT_OVERLAY_HALIGN_LEFT: align text left
+ * @GST_TEXT_OVERLAY_HALIGN_CENTER: align text center
+ * @GST_TEXT_OVERLAY_HALIGN_RIGHT: align text right
+ *
+ * Horizontal alignment of the text.
+ */
+typedef enum {
     GST_TEXT_OVERLAY_HALIGN_LEFT,
     GST_TEXT_OVERLAY_HALIGN_CENTER,
     GST_TEXT_OVERLAY_HALIGN_RIGHT
-};
+} GstTextOverlayHAlign;
 
-enum _GstTextOverlayWrapMode {
+/**
+ * GstTextOverlayWrapMode:
+ * @GST_TEXT_OVERLAY_WRAP_MODE_NONE: no wrapping
+ * @GST_TEXT_OVERLAY_WRAP_MODE_WORD: do word wrapping
+ * @GST_TEXT_OVERLAY_WRAP_MODE_CHAR: do char wrapping
+ * @GST_TEXT_OVERLAY_WRAP_MODE_WORD_CHAR: do word and char wrapping
+ *
+ * Whether to wrap the text and if so how.
+ */
+typedef enum {
     GST_TEXT_OVERLAY_WRAP_MODE_NONE = -1,
     GST_TEXT_OVERLAY_WRAP_MODE_WORD = PANGO_WRAP_WORD,
     GST_TEXT_OVERLAY_WRAP_MODE_CHAR = PANGO_WRAP_CHAR,
     GST_TEXT_OVERLAY_WRAP_MODE_WORD_CHAR = PANGO_WRAP_WORD_CHAR
-};
-
+} GstTextOverlayWrapMode;
 
+/**
+ * GstTextOverlay:
+ *
+ * Opaque textoverlay object structure
+ */
 struct _GstTextOverlay {
     GstElement               element;
 
@@ -53,9 +77,13 @@ struct _GstTextOverlay {
     GstPad                  *text_sinkpad;
     GstPad                  *srcpad;
 
-    GstCollectPads          *collect;
-    GstCollectData          *video_collect_data;
-    GstCollectData          *text_collect_data;
+    GstSegment              *segment;
+    GstBuffer               *text_buffer;
+    gboolean                text_linked;
+    gboolean                video_flushing;
+    gboolean                text_flushing;
+  
+    GCond                   *cond;  /* to signal removal of data */
 
     gint                     width;
     gint                     height;
@@ -81,6 +109,8 @@ struct _GstTextOverlay {
     gboolean                 need_render;
 
     gint                     shading_value;  /* for timeoverlay subclass */
+
+    gboolean                 have_pango_markup;
 };
 
 struct _GstTextOverlayClass {
@@ -89,7 +119,6 @@ struct _GstTextOverlayClass {
     PangoContext *pango_context;
 
     gchar *     (*get_text) (GstTextOverlay *overlay, GstBuffer *video_frame);
-
 };
 
 GType gst_text_overlay_get_type(void) G_GNUC_CONST;
