@@ -58,13 +58,12 @@ enum
   ARG_CROP
 };
 
-static GstElementDetails theora_dec_details = {
-  "TheoraDec",
-  "Codec/Decoder/Video",
-  "decode raw theora streams to raw YUV video",
-  "Benjamin Otte <in7y118@public.uni-hamburg.de>, "
-      "Wim Taymans <wim@fluendo.com>",
-};
+static const GstElementDetails theora_dec_details =
+GST_ELEMENT_DETAILS ("Theora video decoder",
+    "Codec/Decoder/Video",
+    "decode raw theora streams to raw YUV video",
+    "Benjamin Otte <in7y118@public.uni-hamburg.de>, "
+    "Wim Taymans <wim@fluendo.com>");
 
 static GstStaticPadTemplate theora_dec_src_factory =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -173,6 +172,8 @@ gst_theora_dec_reset (GstTheoraDec * dec)
   dec->need_keyframe = TRUE;
   dec->last_timestamp = -1;
   dec->granulepos = -1;
+  dec->discont = TRUE;
+  dec->frame_nr = -1;
   gst_segment_init (&dec->segment, GST_FORMAT_TIME);
 
   GST_OBJECT_LOCK (dec);
@@ -181,7 +182,6 @@ gst_theora_dec_reset (GstTheoraDec * dec)
   GST_OBJECT_UNLOCK (dec);
 }
 
-/* FIXME: copy from libtheora, theora should somehow make this available for seeking */
 static int
 _theora_ilog (unsigned int v)
 {
@@ -282,6 +282,8 @@ theora_get_query_types (GstPad * pad)
 {
   static const GstQueryType theora_src_query_types[] = {
     GST_QUERY_POSITION,
+    GST_QUERY_DURATION,
+    GST_QUERY_CONVERT,
     0
   };
 
@@ -566,6 +568,7 @@ theora_dec_src_event (GstPad * pad, GstEvent * event)
 
       gst_event_parse_seek (event, &rate, &format, &flags, &cur_type, &cur,
           &stop_type, &stop);
+      gst_event_unref (event);
 
       /* we have to ask our peer to seek to time here as we know
        * nothing about how to generate a granulepos from the src
@@ -585,7 +588,6 @@ theora_dec_src_event (GstPad * pad, GstEvent * event)
 
       res = gst_pad_push_event (dec->sinkpad, real_seek);
 
-      gst_event_unref (event);
       break;
     }
     case GST_EVENT_QOS:
@@ -598,17 +600,21 @@ theora_dec_src_event (GstPad * pad, GstEvent * event)
 
       /* we cannot randomly skip frame decoding since we don't have
        * B frames. we can however use the timestamp and diff to not
-       * push late frames. */
+       * push late frames. This would at least save us the time to
+       * crop/memcpy the data. */
       GST_OBJECT_LOCK (dec);
       dec->proportion = proportion;
       dec->earliest_time = timestamp + diff;
       GST_OBJECT_UNLOCK (dec);
 
-      res = gst_pad_event_default (pad, event);
+      GST_DEBUG_OBJECT (dec, "got QoS %" GST_TIME_FORMAT ", %" G_GINT64_FORMAT,
+          GST_TIME_ARGS (timestamp), diff);
+
+      res = gst_pad_push_event (dec->sinkpad, event);
       break;
     }
     default:
-      res = gst_pad_event_default (pad, event);
+      res = gst_pad_push_event (dec->sinkpad, event);
       break;
   }
 done:
@@ -620,7 +626,6 @@ done:
 convert_error:
   {
     GST_DEBUG_OBJECT (dec, "could not convert format");
-    gst_event_unref (event);
     goto done;
   }
 }
@@ -683,19 +688,17 @@ done:
 newseg_wrong_format:
   {
     GST_DEBUG_OBJECT (dec, "received non TIME newsegment");
+    gst_event_unref (event);
     goto done;
   }
 newseg_wrong_rate:
   {
     GST_DEBUG_OBJECT (dec, "negative rates not supported yet");
+    gst_event_unref (event);
     goto done;
   }
 }
 
-#define ROUND_UP_2(x) (((x) + 1) & ~1)
-#define ROUND_UP_4(x) (((x) + 3) & ~3)
-#define ROUND_UP_8(x) (((x) + 7) & ~7)
-
 static GstFlowReturn
 theora_handle_comment_packet (GstTheoraDec * dec, ogg_packet * packet)
 {
@@ -771,9 +774,10 @@ theora_handle_type_packet (GstTheoraDec * dec, ogg_packet * packet)
   if (dec->crop) {
     /* add black borders to make width/height/offsets even. we need this because
      * we cannot express an offset to the peer plugin. */
-    dec->width = ROUND_UP_2 (dec->info.frame_width + (dec->info.offset_x & 1));
+    dec->width =
+        GST_ROUND_UP_2 (dec->info.frame_width + (dec->info.offset_x & 1));
     dec->height =
-        ROUND_UP_2 (dec->info.frame_height + (dec->info.offset_y & 1));
+        GST_ROUND_UP_2 (dec->info.frame_height + (dec->info.offset_y & 1));
     dec->offset_x = dec->info.offset_x & ~1;
     dec->offset_y = dec->info.offset_y & ~1;
   } else {
@@ -871,6 +875,11 @@ theora_dec_push (GstTheoraDec * dec, GstBuffer * buf)
 
         GST_DEBUG_OBJECT (dec, "patch buffer %lld %lld", size, time);
         GST_BUFFER_TIMESTAMP (buffer) = time;
+
+        if (dec->discont) {
+          GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DISCONT);
+          dec->discont = FALSE;
+        }
         /* ignore the result.. */
         gst_pad_push (dec->srcpad, buffer);
         size--;
@@ -878,6 +887,10 @@ theora_dec_push (GstTheoraDec * dec, GstBuffer * buf)
       g_list_free (dec->queued);
       dec->queued = NULL;
     }
+    if (dec->discont) {
+      GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
+      dec->discont = FALSE;
+    }
     result = gst_pad_push (dec->srcpad, buf);
   }
 
@@ -899,31 +912,39 @@ theora_handle_data_packet (GstTheoraDec * dec, ogg_packet * packet,
   gint cwidth, cheight;
   GstFlowReturn result;
 
-  if (!dec->have_header)
+  if (G_UNLIKELY (!dec->have_header))
     goto not_initialized;
 
   /* the second most significant bit of the first data byte is cleared 
    * for keyframes */
   keyframe = (packet->packet[0] & 0x40) == 0;
   if (keyframe) {
+    GST_DEBUG_OBJECT (dec, "we have a keyframe");
     dec->need_keyframe = FALSE;
   } else if (dec->need_keyframe) {
     goto dropping;
   }
 
+  GST_DEBUG_OBJECT (dec, "parsing data packet");
+
   /* this does the decoding */
-  if (theora_decode_packetin (&dec->state, packet))
+  if (G_UNLIKELY (theora_decode_packetin (&dec->state, packet)))
     goto decode_error;
 
   if (outtime != -1) {
     gboolean need_skip;
+    GstClockTime qostime;
+
+    /* qos needs to be done on running time */
+    qostime = gst_segment_to_running_time (&dec->segment, GST_FORMAT_TIME,
+        outtime);
 
     GST_OBJECT_LOCK (dec);
     /* check for QoS, don't perform the last steps of getting and
      * pushing the buffers that are known to be late. */
     /* FIXME, we can also entirely skip decoding if the next valid buffer is 
      * known to be after a keyframe (using the granule_shift) */
-    need_skip = dec->earliest_time != -1 && outtime <= dec->earliest_time;
+    need_skip = dec->earliest_time != -1 && qostime <= dec->earliest_time;
     GST_OBJECT_UNLOCK (dec);
 
     if (need_skip)
@@ -932,10 +953,11 @@ theora_handle_data_packet (GstTheoraDec * dec, ogg_packet * packet,
 
   /* this does postprocessing and set up the decoded frame
    * pointers in our yuv variable */
-  if (theora_decode_YUVout (&dec->state, &yuv) < 0)
+  if (G_UNLIKELY (theora_decode_YUVout (&dec->state, &yuv) < 0))
     goto no_yuv;
 
-  if ((yuv.y_width != dec->info.width) || (yuv.y_height != dec->info.height))
+  if (G_UNLIKELY ((yuv.y_width != dec->info.width)
+          || (yuv.y_height != dec->info.height)))
     goto wrong_dimensions;
 
   width = dec->width;
@@ -946,8 +968,8 @@ theora_handle_data_packet (GstTheoraDec * dec, ogg_packet * packet,
   /* should get the stride from the caps, for now we round up to the nearest
    * multiple of 4 because some element needs it. chroma needs special 
    * treatment, see videotestsrc. */
-  stride_y = ROUND_UP_4 (width);
-  stride_uv = ROUND_UP_8 (width) / 2;
+  stride_y = GST_ROUND_UP_4 (width);
+  stride_uv = GST_ROUND_UP_8 (width) / 2;
 
   out_size = stride_y * height + stride_uv * cheight * 2;
 
@@ -956,7 +978,7 @@ theora_handle_data_packet (GstTheoraDec * dec, ogg_packet * packet,
   result =
       gst_pad_alloc_buffer_and_set_caps (dec->srcpad, GST_BUFFER_OFFSET_NONE,
       out_size, GST_PAD_CAPS (dec->srcpad), &out);
-  if (result != GST_FLOW_OK)
+  if (G_UNLIKELY (result != GST_FLOW_OK))
     goto no_buffer;
 
   /* copy the visible region to the destination. This is actually pretty
@@ -970,7 +992,7 @@ theora_handle_data_packet (GstTheoraDec * dec, ogg_packet * packet,
     guchar *dest_y, *src_y;
     guchar *dest_u, *src_u;
     guchar *dest_v, *src_v;
-    guint offset;
+    gint offset;
 
     dest_y = GST_BUFFER_DATA (out);
     dest_u = dest_y + stride_y * height;
@@ -1001,9 +1023,9 @@ theora_handle_data_packet (GstTheoraDec * dec, ogg_packet * packet,
     }
   }
 
-  /* FIXME, frame_nr not correct */
   GST_BUFFER_OFFSET (out) = dec->frame_nr;
-  dec->frame_nr++;
+  if (dec->frame_nr != -1)
+    dec->frame_nr++;
   GST_BUFFER_OFFSET_END (out) = dec->frame_nr;
   GST_BUFFER_DURATION (out) =
       gst_util_uint64_scale_int (GST_SECOND, dec->info.fps_denominator,
@@ -1024,11 +1046,14 @@ not_initialized:
 dropping:
   {
     GST_WARNING_OBJECT (dec, "dropping frame because we need a keyframe");
+    dec->discont = TRUE;
     return GST_FLOW_OK;
   }
 dropping_qos:
   {
-    dec->frame_nr++;
+    if (dec->frame_nr != -1)
+      dec->frame_nr++;
+    dec->discont = TRUE;
     GST_WARNING_OBJECT (dec, "dropping frame because of QoS");
     return GST_FLOW_OK;
   }
@@ -1069,9 +1094,11 @@ theora_dec_chain (GstPad * pad, GstBuffer * buf)
 
   /* resync on DISCONT */
   if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DISCONT))) {
+    GST_DEBUG_OBJECT (dec, "received DISCONT buffer");
     dec->need_keyframe = TRUE;
     dec->last_timestamp = -1;
     dec->granulepos = -1;
+    dec->discont = TRUE;
   }
 
   /* make ogg_packet out of the buffer */
@@ -1094,7 +1121,10 @@ theora_dec_chain (GstPad * pad, GstBuffer * buf)
     dec->last_timestamp = -1;
   }
 
-  GST_DEBUG_OBJECT (dec, "header=%d packetno=%lld, outtime=%" GST_TIME_FORMAT,
+  if (packet.bytes < 1)
+    goto wrong_size;
+
+  GST_DEBUG_OBJECT (dec, "header=%02x packetno=%lld, outtime=%" GST_TIME_FORMAT,
       packet.packet[0], packet.packetno, GST_TIME_ARGS (dec->last_timestamp));
 
   /* switch depending on packet type */
@@ -1109,7 +1139,7 @@ theora_dec_chain (GstPad * pad, GstBuffer * buf)
   }
 
 done:
-  /* interpollate granule pos */
+  /* interpolate granule pos */
   dec->granulepos = _inc_granulepos (dec, dec->granulepos);
 
   gst_object_unref (dec);
@@ -1117,6 +1147,15 @@ done:
   gst_buffer_unref (buf);
 
   return result;
+
+  /* ERRORS */
+wrong_size:
+  {
+    GST_WARNING_OBJECT (dec, "received empty packet");
+    dec->discont = TRUE;
+    result = GST_FLOW_OK;
+    goto done;
+  }
 }
 
 static GstStateChangeReturn
@@ -1150,6 +1189,7 @@ theora_dec_change_state (GstElement * element, GstStateChange transition)
       theora_clear (&dec->state);
       theora_comment_clear (&dec->comment);
       theora_info_clear (&dec->info);
+      gst_theora_dec_reset (dec);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       break;
