@@ -54,24 +54,11 @@
 GST_DEBUG_CATEGORY_EXTERN (vorbisdec_debug);
 #define GST_CAT_DEFAULT vorbisdec_debug
 
-static GstElementDetails vorbis_dec_details = {
-  "VorbisDec",
-  "Codec/Decoder/Audio",
-  "decode raw vorbis streams to float audio",
-  "Benjamin Otte <in7y118@public.uni-hamburg.de>",
-};
-
-/* Filter signals and args */
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
-
-enum
-{
-  ARG_0
-};
+static const GstElementDetails vorbis_dec_details =
+GST_ELEMENT_DETAILS ("Vorbis audio decoder",
+    "Codec/Decoder/Audio",
+    "decode raw vorbis streams to float audio",
+    "Benjamin Otte <in7y118@public.uni-hamburg.de>");
 
 static GstStaticPadTemplate vorbis_dec_src_factory =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -97,16 +84,12 @@ GST_STATIC_PAD_TEMPLATE ("sink",
 
 GST_BOILERPLATE (GstVorbisDec, gst_vorbis_dec, GstElement, GST_TYPE_ELEMENT);
 
-static void vorbisdec_finalize (GObject * object);
+static void vorbis_dec_finalize (GObject * object);
 static gboolean vorbis_dec_sink_event (GstPad * pad, GstEvent * event);
 static GstFlowReturn vorbis_dec_chain (GstPad * pad, GstBuffer * buffer);
 static GstStateChangeReturn vorbis_dec_change_state (GstElement * element,
     GstStateChange transition);
 
-#if 0
-static const GstFormat *vorbis_dec_get_formats (GstPad * pad);
-#endif
-
 static gboolean vorbis_dec_src_event (GstPad * pad, GstEvent * event);
 static gboolean vorbis_dec_src_query (GstPad * pad, GstQuery * query);
 static gboolean vorbis_dec_convert (GstPad * pad,
@@ -136,45 +119,11 @@ gst_vorbis_dec_class_init (GstVorbisDecClass * klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
 
-  gobject_class->finalize = vorbisdec_finalize;
+  gobject_class->finalize = vorbis_dec_finalize;
 
-  gstelement_class->change_state = vorbis_dec_change_state;
+  gstelement_class->change_state = GST_DEBUG_FUNCPTR (vorbis_dec_change_state);
 }
 
-#if 0
-static const GstFormat *
-vorbis_dec_get_formats (GstPad * pad)
-{
-  static GstFormat src_formats[] = {
-    GST_FORMAT_BYTES,
-    GST_FORMAT_DEFAULT,         /* samples in the audio case */
-    GST_FORMAT_TIME,
-    0
-  };
-  static GstFormat sink_formats[] = {
-    /*GST_FORMAT_BYTES, */
-    GST_FORMAT_TIME,
-    GST_FORMAT_DEFAULT,         /* granulepos or samples */
-    0
-  };
-
-  return (GST_PAD_IS_SRC (pad) ? src_formats : sink_formats);
-}
-#endif
-
-#if 0
-static const GstEventMask *
-vorbis_get_event_masks (GstPad * pad)
-{
-  static const GstEventMask vorbis_dec_src_event_masks[] = {
-    {GST_EVENT_SEEK, GST_SEEK_METHOD_SET | GST_SEEK_FLAG_FLUSH},
-    {0,}
-  };
-
-  return vorbis_dec_src_event_masks;
-}
-#endif
-
 static const GstQueryType *
 vorbis_get_query_types (GstPad * pad)
 {
@@ -192,17 +141,23 @@ gst_vorbis_dec_init (GstVorbisDec * dec, GstVorbisDecClass * g_class)
   dec->sinkpad = gst_pad_new_from_static_template (&vorbis_dec_sink_factory,
       "sink");
 
-  gst_pad_set_event_function (dec->sinkpad, vorbis_dec_sink_event);
-  gst_pad_set_chain_function (dec->sinkpad, vorbis_dec_chain);
-  gst_pad_set_query_function (dec->sinkpad, vorbis_dec_sink_query);
+  gst_pad_set_event_function (dec->sinkpad,
+      GST_DEBUG_FUNCPTR (vorbis_dec_sink_event));
+  gst_pad_set_chain_function (dec->sinkpad,
+      GST_DEBUG_FUNCPTR (vorbis_dec_chain));
+  gst_pad_set_query_function (dec->sinkpad,
+      GST_DEBUG_FUNCPTR (vorbis_dec_sink_query));
   gst_element_add_pad (GST_ELEMENT (dec), dec->sinkpad);
 
   dec->srcpad = gst_pad_new_from_static_template (&vorbis_dec_src_factory,
       "src");
 
-  gst_pad_set_event_function (dec->srcpad, vorbis_dec_src_event);
-  gst_pad_set_query_type_function (dec->srcpad, vorbis_get_query_types);
-  gst_pad_set_query_function (dec->srcpad, vorbis_dec_src_query);
+  gst_pad_set_event_function (dec->srcpad,
+      GST_DEBUG_FUNCPTR (vorbis_dec_src_event));
+  gst_pad_set_query_type_function (dec->srcpad,
+      GST_DEBUG_FUNCPTR (vorbis_get_query_types));
+  gst_pad_set_query_function (dec->srcpad,
+      GST_DEBUG_FUNCPTR (vorbis_dec_src_query));
   gst_pad_use_fixed_caps (dec->srcpad);
   gst_element_add_pad (GST_ELEMENT (dec), dec->srcpad);
 
@@ -210,7 +165,7 @@ gst_vorbis_dec_init (GstVorbisDec * dec, GstVorbisDecClass * g_class)
 }
 
 static void
-vorbisdec_finalize (GObject * object)
+vorbis_dec_finalize (GObject * object)
 {
   /* Release any possibly allocated libvorbis data.
    * _clear functions can safely be called multiple times
@@ -320,7 +275,7 @@ vorbis_dec_src_query (GstPad * pad, GstQuery * query)
                   &value)))
         goto error;
 
-      value = (value - dec->segment_start) + dec->segment_time;
+      value = (value - dec->segment.start) + dec->segment.time;
 
       gst_query_set_position (query, format, value);
 
@@ -471,25 +426,18 @@ vorbis_dec_sink_event (GstPad * pad, GstEvent * event)
       gst_event_parse_new_segment (event, &update, &rate, &format, &start,
           &stop, &time);
 
+      /* we need time and a positive rate for now */
       if (format != GST_FORMAT_TIME)
         goto newseg_wrong_format;
 
       if (rate <= 0.0)
         goto newseg_wrong_rate;
 
-      /* now copy over the values */
-      dec->segment_rate = rate;
-      dec->segment_start = start;
-      dec->segment_stop = stop;
-      dec->segment_time = time;
+      /* now configure the values */
+      gst_segment_set_newsegment (&dec->segment, update,
+          rate, format, start, stop, time);
 
-      dec->granulepos = -1;
-      dec->cur_timestamp = GST_CLOCK_TIME_NONE;
-      dec->prev_timestamp = GST_CLOCK_TIME_NONE;
-
-#ifdef HAVE_VORBIS_SYNTHESIS_RESTART
-      vorbis_synthesis_restart (&dec->vd);
-#endif
+      /* and forward */
       ret = gst_pad_push_event (dec->srcpad, event);
       break;
     }
@@ -704,6 +652,8 @@ header_read_error:
   }
 }
 
+/* These samples can be outside of the float -1.0 -- 1.0 range, this
+ * is allowed, downstream elements are supposed to clip */
 static void
 copy_samples (float *out, float **in, guint samples, gint channels)
 {
@@ -759,11 +709,19 @@ vorbis_dec_push (GstVorbisDec * dec, GstBuffer * buf)
         GstBuffer *buffer = GST_BUFFER (walk->data);
 
         /* ignore the result */
+        if (dec->discont) {
+          GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_DISCONT);
+          dec->discont = FALSE;
+        }
         gst_pad_push (dec->srcpad, buffer);
       }
       g_list_free (dec->queued);
       dec->queued = NULL;
     }
+    if (dec->discont) {
+      GST_BUFFER_FLAG_SET (buf, GST_BUFFER_FLAG_DISCONT);
+      dec->discont = FALSE;
+    }
     result = gst_pad_push (dec->srcpad, buf);
   }
 
@@ -886,12 +844,18 @@ vorbis_dec_chain (GstPad * pad, GstBuffer * buffer)
   ogg_packet packet;
   GstFlowReturn result = GST_FLOW_OK;
 
-  vd = GST_VORBIS_DEC (GST_PAD_PARENT (pad));
+  vd = GST_VORBIS_DEC (gst_pad_get_parent (pad));
 
-  if (GST_BUFFER_SIZE (buffer) == 0) {
-    gst_buffer_unref (buffer);
-    GST_ELEMENT_ERROR (vd, STREAM, DECODE, (NULL), ("empty buffer received"));
-    return GST_FLOW_ERROR;
+  /* resync on DISCONT */
+  if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buffer, GST_BUFFER_FLAG_DISCONT))) {
+    GST_DEBUG_OBJECT (vd, "received DISCONT buffer");
+    vd->granulepos = -1;
+    vd->cur_timestamp = GST_CLOCK_TIME_NONE;
+    vd->prev_timestamp = GST_CLOCK_TIME_NONE;
+#ifdef HAVE_VORBIS_SYNTHESIS_RESTART
+    vorbis_synthesis_restart (&vd->vd);
+#endif
+    vd->discont = TRUE;
   }
 
   /* only ogg has granulepos, demuxers of other container formats 
@@ -921,6 +885,9 @@ vorbis_dec_chain (GstPad * pad, GstBuffer * buffer)
    */
   packet.e_o_s = 0;
 
+  if (packet.bytes < 1)
+    goto wrong_size;
+
   GST_DEBUG_OBJECT (vd, "vorbis granule: %" G_GINT64_FORMAT,
       (gint64) packet.granulepos);
 
@@ -940,8 +907,18 @@ vorbis_dec_chain (GstPad * pad, GstBuffer * buffer)
 
 done:
   gst_buffer_unref (buffer);
+  gst_object_unref (vd);
 
   return result;
+
+  /* ERRORS */
+wrong_size:
+  {
+    GST_ELEMENT_ERROR (vd, STREAM, DECODE, (NULL), ("empty buffer received"));
+    result = GST_FLOW_ERROR;
+    vd->discont = TRUE;
+    goto done;
+  }
 }
 
 static GstStateChangeReturn
@@ -962,6 +939,7 @@ vorbis_dec_change_state (GstElement * element, GstStateChange transition)
       vd->prev_timestamp = GST_CLOCK_TIME_NONE;
       vd->granulepos = -1;
       vd->packetno = 0;
+      vd->discont = TRUE;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       break;
