@@ -33,10 +33,10 @@ G_BEGIN_DECLS
 #define GST_VORBIS_DEC(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_VORBIS_DEC,GstVorbisDec))
 #define GST_VORBIS_DEC_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VORBIS_DEC,GstVorbisDec))
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VORBIS_DEC,GstVorbisDecClass))
 #define GST_IS_VORBIS_DEC(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_VORBIS_DEC))
-#define GST_IS_VORBIS_DEC_CLASS(obj) \
+#define GST_IS_VORBIS_DEC_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_VORBIS_DEC))
 
 typedef struct _GstVorbisDec GstVorbisDec;
@@ -64,10 +64,8 @@ struct _GstVorbisDec {
 
   GList                 *queued;
 
-  gdouble segment_rate;
-  gint64  segment_start;
-  gint64  segment_stop;
-  gint64  segment_time;
+  GstSegment		segment;
+  gboolean		discont;
 
   GstClockTime          cur_timestamp; /* only used with non-ogg container formats */
   GstClockTime          prev_timestamp; /* only used with non-ogg container formats */
