@@ -118,7 +118,7 @@ gst_audioringbuffer_class_init (GstAudioRingBufferClass * klass)
   gstobject_class = (GstObjectClass *) klass;
   gstringbuffer_class = (GstRingBufferClass *) klass;
 
-  ring_parent_class = g_type_class_ref (GST_TYPE_RING_BUFFER);
+  ring_parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_audioringbuffer_dispose);
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_audioringbuffer_finalize);
@@ -465,7 +465,6 @@ gst_audio_src_class_init (GstAudioSrcClass * klass)
 static void
 gst_audio_src_init (GstAudioSrc * audiosrc, GstAudioSrcClass * g_class)
 {
-  gst_base_src_set_live (GST_BASE_SRC (audiosrc), TRUE);
 }
 
 static GstRingBuffer *
