@@ -34,10 +34,14 @@ enum
   LAST_SIGNAL
 };
 
-/* we tollerate a 10th of a second diff before we start resyncing. This
+/* we tollerate half a second diff before we start resyncing. This
  * should be enough to compensate for various rounding errors in the timestamp
- * and sample offset position. */
-#define DIFF_TOLERANCE  10
+ * and sample offset position. 
+ * This is an emergency resync fallback since buffers marked as DISCONT will
+ * always lock to the correct timestamp immediatly and buffers not marked as
+ * DISCONT are contiguous bu definition.
+ */
+#define DIFF_TOLERANCE  2
 
 #define DEFAULT_BUFFER_TIME     200 * GST_USECOND
 #define DEFAULT_LATENCY_TIME    10 * GST_USECOND
@@ -64,6 +68,8 @@ static void gst_base_audio_sink_set_property (GObject * object, guint prop_id,
 static void gst_base_audio_sink_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
+static GstStateChangeReturn gst_base_audio_sink_async_play (GstBaseSink *
+    basesink);
 static GstStateChangeReturn gst_base_audio_sink_change_state (GstElement *
     element, GstStateChange transition);
 
@@ -141,6 +147,8 @@ gst_base_audio_sink_class_init (GstBaseAudioSinkClass * klass)
   gstbasesink_class->get_times =
       GST_DEBUG_FUNCPTR (gst_base_audio_sink_get_times);
   gstbasesink_class->set_caps = GST_DEBUG_FUNCPTR (gst_base_audio_sink_setcaps);
+  gstbasesink_class->async_play =
+      GST_DEBUG_FUNCPTR (gst_base_audio_sink_async_play);
 }
 
 static void
@@ -166,9 +174,10 @@ gst_base_audio_sink_dispose (GObject * object)
     gst_object_unref (sink->provided_clock);
   sink->provided_clock = NULL;
 
-  if (sink->ringbuffer)
-    gst_object_unref (sink->ringbuffer);
-  sink->ringbuffer = NULL;
+  if (sink->ringbuffer) {
+    gst_object_unparent (GST_OBJECT_CAST (sink->ringbuffer));
+    sink->ringbuffer = NULL;
+  }
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
@@ -356,6 +365,12 @@ gst_base_audio_sink_drain (GstBaseAudioSink * sink)
   if (!sink->ringbuffer->spec.rate)
     return TRUE;
 
+  /* need to start playback before we can drain, but only when
+   * we have successfully negotiated a format and thus aqcuired the
+   * ringbuffer. */
+  if (gst_ring_buffer_is_acquired (sink->ringbuffer))
+    gst_ring_buffer_start (sink->ringbuffer);
+
   if (sink->next_sample != -1) {
     GstClockTime time;
     GstClock *clock;
@@ -389,16 +404,16 @@ gst_base_audio_sink_event (GstBaseSink * bsink, GstEvent * event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_FLUSH_START:
-      gst_ring_buffer_set_flushing (sink->ringbuffer, TRUE);
+      if (sink->ringbuffer)
+        gst_ring_buffer_set_flushing (sink->ringbuffer, TRUE);
       break;
     case GST_EVENT_FLUSH_STOP:
       /* always resync on sample after a flush */
       sink->next_sample = -1;
-      gst_ring_buffer_set_flushing (sink->ringbuffer, FALSE);
+      if (sink->ringbuffer)
+        gst_ring_buffer_set_flushing (sink->ringbuffer, FALSE);
       break;
     case GST_EVENT_EOS:
-      /* need to start playback when we reach EOS */
-      gst_ring_buffer_start (sink->ringbuffer);
       /* now wait till we played everything */
       gst_base_audio_sink_drain (sink);
       break;
@@ -576,9 +591,10 @@ gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   if (G_LIKELY (sink->next_sample != -1)) {
     diff = ABS ((gint64) render_offset - (gint64) sink->next_sample);
 
-    /* we tollerate a 10th of a second diff before we start resyncing. This
+    /* we tollerate half a second diff before we start resyncing. This
      * should be enough to compensate for various rounding errors in the timestamp
-     * and sample offset position. We always resync if we got a discont anyway. */
+     * and sample offset position. We always resync if we got a discont anyway and
+     * non-discont should be aligned by definition. */
     if (diff < ringbuf->spec.rate / DIFF_TOLERANCE) {
       GST_DEBUG_OBJECT (sink,
           "align with prev sample, %" G_GINT64_FORMAT " < %lu", diff,
@@ -586,7 +602,9 @@ gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
       /* just align with previous sample then */
       render_offset = sink->next_sample;
     } else {
-      GST_DEBUG_OBJECT (sink,
+      /* timestamps drifted apart from previous samples too much, we need to
+       * resync. */
+      GST_WARNING_OBJECT (sink,
           "resync after discont with previous sample of diff: %lu", diff);
     }
   } else {
@@ -689,6 +707,62 @@ gst_base_audio_sink_callback (GstRingBuffer * rbuf, guint8 * data, guint len,
   //GstBaseAudioSink *sink = GST_BASE_AUDIO_SINK (data);
 }
 
+/* should be called with the LOCK */
+static GstStateChangeReturn
+gst_base_audio_sink_async_play (GstBaseSink * basesink)
+{
+  GstClock *clock;
+  GstClockTime time, base;
+  GstBaseAudioSink *sink;
+
+  sink = GST_BASE_AUDIO_SINK (basesink);
+
+  GST_DEBUG_OBJECT (sink, "ringbuffer may start now");
+  gst_ring_buffer_may_start (sink->ringbuffer, TRUE);
+
+  clock = GST_ELEMENT_CLOCK (sink);
+  if (clock == NULL)
+    goto no_clock;
+
+  /* FIXME, only start slaving when we really start the ringbuffer */
+  /* if we are slaved to a clock, we need to set the initial
+   * calibration */
+  if (clock != sink->provided_clock) {
+    GstClockTime rate_num, rate_denom;
+
+    base = GST_ELEMENT_CAST (sink)->base_time;
+    time = gst_clock_get_internal_time (sink->provided_clock);
+
+    GST_DEBUG_OBJECT (sink,
+        "time: %" GST_TIME_FORMAT " base: %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (time), GST_TIME_ARGS (base));
+
+    /* FIXME, this is not yet accurate enough for smooth playback */
+    gst_clock_get_calibration (sink->provided_clock, NULL, NULL, &rate_num,
+        &rate_denom);
+    /* Does not work yet. */
+    gst_clock_set_calibration (sink->provided_clock, time, base,
+        rate_num, rate_denom);
+
+    gst_clock_set_master (sink->provided_clock, clock);
+  }
+
+no_clock:
+  return GST_STATE_CHANGE_SUCCESS;
+}
+
+static GstStateChangeReturn
+gst_base_audio_sink_do_play (GstBaseAudioSink * sink)
+{
+  GstStateChangeReturn ret;
+
+  GST_OBJECT_LOCK (sink);
+  ret = gst_base_audio_sink_async_play (GST_BASE_SINK_CAST (sink));
+  GST_OBJECT_UNLOCK (sink);
+
+  return ret;
+}
+
 static GstStateChangeReturn
 gst_base_audio_sink_change_state (GstElement * element,
     GstStateChange transition)
@@ -707,48 +781,25 @@ gst_base_audio_sink_change_state (GstElement * element,
         goto open_failed;
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      gst_ring_buffer_set_flushing (sink->ringbuffer, FALSE);
       sink->next_sample = -1;
+      gst_ring_buffer_set_flushing (sink->ringbuffer, FALSE);
+      gst_ring_buffer_may_start (sink->ringbuffer, FALSE);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
-    {
-      GstClock *clock;
-      GstClockTime time, base;
-
+      gst_base_audio_sink_do_play (sink);
+      break;
+    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
+      /* need to take the lock so we don't interfere with an
+       * async play */
       GST_OBJECT_LOCK (sink);
-      clock = GST_ELEMENT_CLOCK (sink);
-      if (clock == NULL)
-        goto no_clock;
-
-      /* FIXME, only start slaving when we really start the ringbuffer */
-      /* if we are slaved to a clock, we need to set the initial
-       * calibration */
-      if (clock != sink->provided_clock) {
-        GstClockTime rate_num, rate_denom;
-
-        base = element->base_time;
-        time = gst_clock_get_internal_time (sink->provided_clock);
-
-        GST_DEBUG_OBJECT (sink,
-            "time: %" GST_TIME_FORMAT " base: %" GST_TIME_FORMAT,
-            GST_TIME_ARGS (time), GST_TIME_ARGS (base));
-
-        gst_clock_set_master (sink->provided_clock, clock);
-        /* FIXME, this is not yet accurate enough for smooth playback */
-        gst_clock_get_calibration (sink->provided_clock, NULL, NULL, &rate_num,
-            &rate_denom);
-        /* Does not work yet. */
-        gst_clock_set_calibration (sink->provided_clock,
-            time, element->base_time, rate_num, rate_denom);
-      }
-    no_clock:
+      /* ringbuffer cannot start anymore */
+      gst_ring_buffer_may_start (sink->ringbuffer, FALSE);
+      gst_ring_buffer_pause (sink->ringbuffer);
       GST_OBJECT_UNLOCK (sink);
       break;
-    }
-    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-      gst_ring_buffer_pause (sink->ringbuffer);
-      break;
     case GST_STATE_CHANGE_PAUSED_TO_READY:
+      /* make sure we unblock before calling the parent state change
+       * so it can grab the STREAM_LOCK */
       gst_ring_buffer_set_flushing (sink->ringbuffer, TRUE);
       break;
     default:
@@ -775,8 +826,10 @@ gst_base_audio_sink_change_state (GstElement * element,
 
   return ret;
 
+  /* ERRORS */
 open_failed:
   {
+    /* subclass must post a meaningfull error message */
     GST_DEBUG_OBJECT (sink, "open failed");
     return GST_STATE_CHANGE_FAILURE;
   }
