@@ -64,7 +64,7 @@
 
 #define M_PI_M2 ( M_PI + M_PI )
 
-static GstElementDetails gst_audio_test_src_details =
+static const GstElementDetails gst_audio_test_src_details =
 GST_ELEMENT_DETAILS ("Audio test source",
     "Source/Audio",
     "Creates audio test signals of given frequency and volume",
@@ -103,7 +103,7 @@ static GType
 gst_audiostestsrc_wave_get_type (void)
 {
   static GType audiostestsrc_wave_type = 0;
-  static GEnumValue audiostestsrc_waves[] = {
+  static const GEnumValue audiostestsrc_waves[] = {
     {GST_AUDIO_TEST_SRC_WAVE_SINE, "Sine", "sine"},
     {GST_AUDIO_TEST_SRC_WAVE_SQUARE, "Square", "square"},
     {GST_AUDIO_TEST_SRC_WAVE_SAW, "Saw", "saw"},
