@@ -128,12 +128,11 @@ static GstElementClass *parent_class;
 
 //static guint gst_play_bin_signals[LAST_SIGNAL] = { 0 };
 
-static GstElementDetails gst_play_bin_details = {
-  "Player Bin",
-  "Generic/Bin/Player",
-  "Autoplug and play media from an uri",
-  "Wim Taymans <wim@fluendo.com>"
-};
+static const GstElementDetails gst_play_bin_details =
+GST_ELEMENT_DETAILS ("Player Bin",
+    "Generic/Bin/Player",
+    "Autoplug and play media from an uri",
+    "Wim Taymans <wim@fluendo.com>");
 
 static GType
 gst_play_bin_get_type (void)
@@ -174,7 +173,7 @@ gst_play_bin_class_init (GstPlayBinClass * klass)
   gstbin_klass = (GstBinClass *) klass;
   playbasebin_klass = (GstPlayBaseBinClass *) klass;
 
-  parent_class = g_type_class_ref (gst_play_base_bin_get_type ());
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_klass->set_property = gst_play_bin_set_property;
   gobject_klass->get_property = gst_play_bin_get_property;
@@ -546,8 +545,9 @@ static void
 handoff (GstElement * identity, GstBuffer * frame, gpointer data)
 {
   GstPlayBin *play_bin = GST_PLAY_BIN (data);
+  GstBuffer **frame_p = &play_bin->frame;
 
-  gst_mini_object_replace ((GstMiniObject **) & play_bin->frame,
+  gst_mini_object_replace ((GstMiniObject **) frame_p,
       GST_MINI_OBJECT_CAST (frame));
 
   /* applications need to know the buffer caps,
@@ -619,9 +619,13 @@ gen_video_element (GstPlayBin * play_bin)
   identity = gst_element_factory_make ("identity", "id");
   g_object_set (identity, "silent", TRUE, NULL);
   g_signal_connect (identity, "handoff", G_CALLBACK (handoff), play_bin);
-  conv = gst_element_factory_make ("ffmpegcolorspace", "vconv");
-  scale = gst_element_factory_make ("videoscale", "vscale");
   gst_bin_add (GST_BIN (element), identity);
+  conv = gst_element_factory_make ("ffmpegcolorspace", "vconv");
+  if (conv == NULL)
+    goto no_colorspace;
+  scale = gst_element_factory_make ("videoscale", "vscale");
+  if (scale == NULL)
+    goto no_videoscale;
   gst_bin_add (GST_BIN (element), conv);
   gst_bin_add (GST_BIN (element), scale);
   gst_bin_add (GST_BIN (element), sink);
@@ -641,6 +645,24 @@ gen_video_element (GstPlayBin * play_bin)
   g_hash_table_insert (play_bin->cache, "vbin", element);
 
   return element;
+
+no_colorspace:
+  {
+    GST_ELEMENT_ERROR (play_bin, CORE, MISSING_PLUGIN,
+        (_("Missing element '%s' - check your GStreamer installation."),
+            "ffmpegcolorspace"), (NULL));
+    gst_object_unref (element);
+    return NULL;
+  }
+
+no_videoscale:
+  {
+    GST_ELEMENT_ERROR (play_bin, CORE, MISSING_PLUGIN,
+        (_("Missing element '%s' - check your GStreamer installation."),
+            "videoscale"), ("possibly a liboil version mismatch?"));
+    gst_object_unref (element);
+    return NULL;
+  }
 }
 
 /* make an element for playback of video with subtitles embedded.
@@ -686,6 +708,7 @@ gen_text_element (GstPlayBin * play_bin)
   /* Take a ref */
   play_bin->textoverlay_element = GST_ELEMENT (gst_object_ref (overlay));
 
+  /* we know this will succeed, as the video bin already created one before */
   csp = gst_element_factory_make ("ffmpegcolorspace", "subtitlecsp");
 
   /* Add our elements */
@@ -728,9 +751,9 @@ gen_audio_element (GstPlayBin * play_bin)
 {
   GstElement *element;
   GstElement *conv;
+  GstElement *scale;
   GstElement *sink;
   GstElement *volume;
-  GstElement *scale;
   GstPad *pad;
 
   element = g_hash_table_lookup (play_bin->cache, "abin");
@@ -739,7 +762,12 @@ gen_audio_element (GstPlayBin * play_bin)
   }
   element = gst_bin_new ("abin");
   conv = gst_element_factory_make ("audioconvert", "aconv");
-  scale = gst_element_factory_make ("audioscale", "ascale");
+  if (conv == NULL)
+    goto no_audioconvert;
+
+  scale = gst_element_factory_make ("audioresample", "aresample");
+  if (scale == NULL)
+    goto no_audioresample;
 
   volume = gst_element_factory_make ("volume", "volume");
   g_object_set (G_OBJECT (volume), "volume", play_bin->volume, NULL);
@@ -764,12 +792,12 @@ gen_audio_element (GstPlayBin * play_bin)
   g_hash_table_insert (play_bin->cache, "audio_sink", sink);
 
   gst_bin_add (GST_BIN (element), conv);
-  //gst_bin_add (GST_BIN (element), scale);
+  gst_bin_add (GST_BIN (element), scale);
   gst_bin_add (GST_BIN (element), volume);
   gst_bin_add (GST_BIN (element), sink);
 
-  gst_element_link_pads (conv, "src",   /*scale, "sink");
-                                           gst_element_link_pads (scale, "src", */ volume, "sink");
+  gst_element_link_pads (conv, "src", scale, "sink");
+  gst_element_link_pads (scale, "src", volume, "sink");
   gst_element_link_pads (volume, "src", sink, "sink");
 
   pad = gst_element_get_pad (conv, "sink");
@@ -784,6 +812,24 @@ gen_audio_element (GstPlayBin * play_bin)
   g_hash_table_insert (play_bin->cache, "abin", element);
 
   return element;
+
+no_audioconvert:
+  {
+    GST_ELEMENT_ERROR (play_bin, CORE, MISSING_PLUGIN,
+        (_("Missing element '%s' - check your GStreamer installation."),
+            "audioconvert"), ("possibly a liboil version mismatch?"));
+    gst_object_unref (element);
+    return NULL;
+  }
+
+no_audioresample:
+  {
+    GST_ELEMENT_ERROR (play_bin, CORE, MISSING_PLUGIN,
+        (_("Missing element '%s' - check your GStreamer installation."),
+            "audioresample"), ("possibly a liboil version mismatch?"));
+    gst_object_unref (element);
+    return NULL;
+  }
 }
 
 /* make the element (bin) that contains the elements needed to perform
@@ -793,21 +839,21 @@ gen_audio_element (GstPlayBin * play_bin)
  * normal video bin. The video bin is run in a thread to make sure it does
  * not block the audio playback pipeline.
  *
- *  +--------------------------------------------------------------------------+
- *  | visbin                                                                   |
- *  |      +------+   +----------------+                                       |
- *  |      | tee  |   |   abin ...     |                                       |
- *  |   +-sink   src-sink              |                                       |
- *  |   |  |      |   +----------------+                 +-------------------+ |
- *  |   |  |      |                                      | vthread           | |
- *  |   |  |      |   +---------+   +------+   +------+  | +--------------+  | |
- *  |   |  |      |   |audioconv|   | vis  |   |vqueue|  | | vbin ...     |  | |
- *  |   |  |     src-sink      src-sink   src-sink   src-sink             |  | |
- *  |   |  |      |   +---------+   +------+   +------+  | +--------------+  | |
- *  |   |  |      |                                      +-------------------+ |
- *  |   |  +------+                                                            |
- * sink-+                                                                      |
-   +--------------------------------------------------------------------------+
+ *  +--------------------------------------------------------------------+
+ *  | visbin                                                             |
+ *  |      +------+   +--------+   +----------------+                    |
+ *  |      | tee  |   | aqueue |   |   abin ...     |                    |
+ *  |   +-sink   src-sink     src-sink              |                    |
+ *  |   |  |      |   +--------+   +----------------+                    |
+ *  |   |  |      |                                                      |
+ *  |   |  |      |   +------+   +---------+   +------+   +-----------+  |
+ *  |   |  |      |   |vqueue|   |audioconv|   | vis  |   | vbin ...  |  |
+ *  |   |  |     src-sink   src-sink      src-sink   src-sink         |  |
+ *  |   |  |      |   +------+   +---------+   +------+   +-----------+  |
+ *  |   |  |      |                                                      |
+ *  |   |  +------+                                                      |
+ * sink-+                                                                |
+   +---------------------------------------------------------------------+
  */
 static GstElement *
 gen_vis_element (GstPlayBin * play_bin)
@@ -843,6 +889,9 @@ gen_vis_element (GstPlayBin * play_bin)
   gst_bin_add (GST_BIN (element), tee);
 
   conv = gst_element_factory_make ("audioconvert", "aconv");
+  if (conv == NULL)
+    goto no_audioconvert;
+
   if (play_bin->visualisation) {
     gst_object_ref (play_bin->visualisation);
     vis = play_bin->visualisation;
@@ -853,10 +902,9 @@ gen_vis_element (GstPlayBin * play_bin)
   gst_bin_add (GST_BIN (element), conv);
   gst_bin_add (GST_BIN (element), vis);
 
+  gst_element_link_pads (vqueue, "src", conv, "sink");
   gst_element_link_pads (conv, "src", vis, "sink");
-  gst_element_link_pads (vis, "src", vqueue, "sink");
-
-  gst_element_link_pads (vqueue, "src", vsink, "sink");
+  gst_element_link_pads (vis, "src", vsink, "sink");
 
   pad = gst_element_get_pad (aqueue, "sink");
   rpad = gst_element_get_request_pad (tee, "src%d");
@@ -865,7 +913,7 @@ gen_vis_element (GstPlayBin * play_bin)
   gst_object_unref (pad);
   gst_element_link_pads (aqueue, "src", asink, "sink");
 
-  pad = gst_element_get_pad (conv, "sink");
+  pad = gst_element_get_pad (vqueue, "sink");
   rpad = gst_element_get_request_pad (tee, "src%d");
   gst_pad_link (rpad, pad);
   gst_object_unref (rpad);
@@ -876,6 +924,15 @@ gen_vis_element (GstPlayBin * play_bin)
   gst_object_unref (pad);
 
   return element;
+
+no_audioconvert:
+  {
+    GST_ELEMENT_ERROR (play_bin, CORE, MISSING_PLUGIN,
+        (_("Missing element '%s' - check your GStreamer installation."),
+            "audioconvert"), ("possibly a liboil version mismatch?"));
+    gst_object_unref (element);
+    return NULL;
+  }
 }
 
 /* get rid of all installed sinks */
@@ -1049,7 +1106,7 @@ link_failed:
     GST_DEBUG_OBJECT (play_bin,
         "link failed when adding sink, caps %s, reason %d", capsstr, linkres);
     g_free (capsstr);
-    g_free (caps);
+    gst_caps_unref (caps);
 
     gst_element_set_state (sink, GST_STATE_NULL);
     gst_bin_remove (GST_BIN (play_bin), sink);
@@ -1067,7 +1124,7 @@ subtitle_failed:
         "subtitle link failed when adding sink, caps %s, reason %d", capsstr,
         linkres);
     g_free (capsstr);
-    g_free (caps);
+    gst_caps_unref (caps);
 
     return TRUE;
   }
@@ -1155,25 +1212,34 @@ setup_sinks (GstPlayBaseBin * play_base_bin, GstPlayBaseGroup * group)
         goto beach;
       }
 
-      ghost = gst_ghost_pad_new ("text_src", textsrcpad);
-      if (!GST_IS_PAD (ghost)) {
-        GST_WARNING_OBJECT (textsrcpad, "failed creating ghost pad for "
-            "subtitle-bin");
-        gst_object_unref (parent);
-        gst_object_unref (grandparent);
-        gst_object_unref (textsrcpad);
-        textsrcpad = NULL;
-        goto beach;
-      }
+      /* We ghost the pad on subtitle_bin only, if the text pad is from the
+         media demuxer we keep it as it is */
+      if (!GST_IS_PLAY_BIN (grandparent)) {
+        GST_DEBUG_OBJECT (textsrcpad, "this subtitle pad is from a subtitle "
+            "file, ghosting to a suitable hierarchy");
+        ghost = gst_ghost_pad_new ("text_src", textsrcpad);
+        if (!GST_IS_PAD (ghost)) {
+          GST_WARNING_OBJECT (textsrcpad, "failed creating ghost pad for "
+              "subtitle-bin");
+          gst_object_unref (parent);
+          gst_object_unref (grandparent);
+          gst_object_unref (textsrcpad);
+          textsrcpad = NULL;
+          goto beach;
+        }
 
-      if (gst_element_add_pad (GST_ELEMENT (grandparent), ghost)) {
-        gst_object_unref (textsrcpad);
-        textsrcpad = ghost;
+        if (gst_element_add_pad (GST_ELEMENT (grandparent), ghost)) {
+          gst_object_unref (textsrcpad);
+          textsrcpad = gst_object_ref (ghost);
+        } else {
+          GST_WARNING_OBJECT (ghost, "failed adding ghost pad on subtitle-bin");
+          gst_object_unref (ghost);
+          gst_object_unref (textsrcpad);
+          textsrcpad = NULL;
+        }
       } else {
-        GST_WARNING_OBJECT (ghost, "failed adding ghost pad on subtitle-bin");
-        gst_object_unref (ghost);
-        gst_object_unref (textsrcpad);
-        textsrcpad = NULL;
+        GST_DEBUG_OBJECT (textsrcpad, "this subtitle pad is from the demuxer "
+            "no changes to hierarchy needed");
       }
 
       gst_object_unref (parent);
@@ -1188,6 +1254,9 @@ setup_sinks (GstPlayBaseBin * play_base_bin, GstPlayBaseGroup * group)
         "src");
     res = add_sink (play_bin, sink, pad, textsrcpad);
     gst_object_unref (pad);
+    if (textsrcpad) {
+      gst_object_unref (textsrcpad);
+    }
   }
 
   /* remove the sinks now, pipeline get_state will now wait for the
@@ -1214,8 +1283,13 @@ gst_play_bin_send_event_to_sink (GstPlayBin * play_bin, GstEvent * event)
     GstElement *sink = GST_ELEMENT_CAST (sinks->data);
 
     gst_event_ref (event);
-    if ((res = gst_element_send_event (sink, event)))
+    if ((res = gst_element_send_event (sink, event))) {
+      GST_DEBUG_OBJECT (play_bin,
+          "Sent event succesfully to sink %" GST_PTR_FORMAT, sink);
       break;
+    }
+    GST_DEBUG_OBJECT (play_bin,
+        "Event failed when sent to sink %" GST_PTR_FORMAT, sink);
 
     sinks = g_list_next (sinks);
   }
@@ -1248,14 +1322,18 @@ do_playbin_seek (GstElement * element, GstEvent * event)
 
     if (was_playing) {
       gst_element_set_state (element, GST_STATE_PAUSED);
+      gst_element_get_state (element, NULL, NULL, 50 * GST_MSECOND);
     }
   }
 
+  GST_DEBUG_OBJECT (element, "Sending seek event to a sink");
   res = gst_play_bin_send_event_to_sink (GST_PLAY_BIN (element), event);
 
-  if (flush && res) {
+  if (flush) {
     /* need to reset the stream time to 0 after a flushing seek */
-    gst_pipeline_set_new_stream_time (GST_PIPELINE (element), 0);
+    if (res)
+      gst_pipeline_set_new_stream_time (GST_PIPELINE (element), 0);
+
     if (was_playing)
       /* and continue playing */
       gst_element_set_state (element, GST_STATE_PLAYING);
