@@ -18,20 +18,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-/**
- * SECTION:element-tcpclientsrc
- * @see_also: #tcpclientsink
- *
- * <refsect2>
- * <title>Example launch line</title>
- * |[
- * # server:
- * nc -l -p 3000
- * # client:
- * gst-launch tcpclientsrc protocol=none port=3000 ! fdsink fd=2
- * ]| everything you type in the server is shown on the client
- * </refsect2>
- */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -46,12 +32,36 @@
 #include <fcntl.h>
 
 
-GST_DEBUG_CATEGORY_STATIC (tcpclientsrc_debug);
+/* control stuff stolen from fdsrc */
+#define CONTROL_STOP            'S'     /* stop the select call */
+#define CONTROL_SOCKETS(o)      o->control_fds
+#define WRITE_SOCKET(o)         o->control_fds[1]
+#define READ_SOCKET(o)          o->control_fds[0]
+
+#define SEND_COMMAND(o, command)          \
+G_STMT_START {                              \
+  unsigned char c; c = command;             \
+  write (WRITE_SOCKET(o), &c, 1);         \
+} G_STMT_END
+
+#define READ_COMMAND(o, command, res)        \
+G_STMT_START {                                 \
+  res = read(READ_SOCKET(o), &command, 1);   \
+} G_STMT_END
+
+
+GST_DEBUG_CATEGORY (tcpclientsrc_debug);
 #define GST_CAT_DEFAULT tcpclientsrc_debug
 
 #define MAX_READ_SIZE                   4 * 1024
 
 
+static const GstElementDetails gst_tcp_client_src_details =
+GST_ELEMENT_DETAILS ("TCP client source",
+    "Source/Network",
+    "Receive data as a client over the network via TCP",
+    "Thomas Vander Stichele <thomas at apestaart dot org>");
+
 static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -80,7 +90,6 @@ static GstFlowReturn gst_tcp_client_src_create (GstPushSrc * psrc,
 static gboolean gst_tcp_client_src_stop (GstBaseSrc * bsrc);
 static gboolean gst_tcp_client_src_start (GstBaseSrc * bsrc);
 static gboolean gst_tcp_client_src_unlock (GstBaseSrc * bsrc);
-static gboolean gst_tcp_client_src_unlock_stop (GstBaseSrc * bsrc);
 
 static void gst_tcp_client_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -93,12 +102,10 @@ gst_tcp_client_src_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class, &srctemplate);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&srctemplate));
 
-  gst_element_class_set_details_simple (element_class,
-      "TCP client source", "Source/Network",
-      "Receive data as a client over the network via TCP",
-      "Thomas Vander Stichele <thomas at apestaart dot org>");
+  gst_element_class_set_details (element_class, &gst_tcp_client_src_details);
 }
 
 static void
@@ -116,24 +123,21 @@ gst_tcp_client_src_class_init (GstTCPClientSrcClass * klass)
   gobject_class->get_property = gst_tcp_client_src_get_property;
   gobject_class->finalize = gst_tcp_client_src_finalize;
 
-  g_object_class_install_property (gobject_class, PROP_HOST,
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_HOST,
       g_param_spec_string ("host", "Host",
           "The host IP address to receive packets from", TCP_DEFAULT_HOST,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class, PROP_PORT,
+          G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PORT,
       g_param_spec_int ("port", "Port", "The port to receive packets from", 0,
-          TCP_HIGHEST_PORT, TCP_DEFAULT_PORT,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          TCP_HIGHEST_PORT, TCP_DEFAULT_PORT, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_PROTOCOL,
       g_param_spec_enum ("protocol", "Protocol", "The protocol to wrap data in",
-          GST_TYPE_TCP_PROTOCOL, GST_TCP_PROTOCOL_NONE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          GST_TYPE_TCP_PROTOCOL, GST_TCP_PROTOCOL_NONE, G_PARAM_READWRITE));
 
   gstbasesrc_class->get_caps = gst_tcp_client_src_getcaps;
   gstbasesrc_class->start = gst_tcp_client_src_start;
   gstbasesrc_class->stop = gst_tcp_client_src_stop;
   gstbasesrc_class->unlock = gst_tcp_client_src_unlock;
-  gstbasesrc_class->unlock_stop = gst_tcp_client_src_unlock_stop;
 
   gstpush_src_class->create = gst_tcp_client_src_create;
 
@@ -146,10 +150,15 @@ gst_tcp_client_src_init (GstTCPClientSrc * this, GstTCPClientSrcClass * g_class)
 {
   this->port = TCP_DEFAULT_PORT;
   this->host = g_strdup (TCP_DEFAULT_HOST);
-  this->sock_fd.fd = -1;
+  this->sock_fd = -1;
   this->protocol = GST_TCP_PROTOCOL_NONE;
   this->caps = NULL;
 
+  READ_SOCKET (this) = -1;
+  WRITE_SOCKET (this) = -1;
+
+  gst_base_src_set_live (GST_BASE_SRC (this), TRUE);
+
   GST_OBJECT_FLAG_UNSET (this, GST_TCP_CLIENT_SRC_OPEN);
 }
 
@@ -159,8 +168,6 @@ gst_tcp_client_src_finalize (GObject * gobject)
   GstTCPClientSrc *this = GST_TCP_CLIENT_SRC (gobject);
 
   g_free (this->host);
-
-  G_OBJECT_CLASS (parent_class)->finalize (gobject);
 }
 
 static GstCaps *
@@ -198,8 +205,8 @@ gst_tcp_client_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
   /* read the buffer header if we're using a protocol */
   switch (src->protocol) {
     case GST_TCP_PROTOCOL_NONE:
-      ret = gst_tcp_read_buffer (GST_ELEMENT (src), src->sock_fd.fd,
-          src->fdset, outbuf);
+      ret = gst_tcp_read_buffer (GST_ELEMENT (src), src->sock_fd,
+          READ_SOCKET (src), outbuf);
       break;
 
     case GST_TCP_PROTOCOL_GDP:
@@ -208,8 +215,8 @@ gst_tcp_client_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
         GstCaps *caps;
 
         GST_DEBUG_OBJECT (src, "getting caps through GDP");
-        ret = gst_tcp_gdp_read_caps (GST_ELEMENT (src), src->sock_fd.fd,
-            src->fdset, &caps);
+        ret = gst_tcp_gdp_read_caps (GST_ELEMENT (src), src->sock_fd,
+            READ_SOCKET (src), &caps);
 
         if (ret != GST_FLOW_OK)
           goto no_caps;
@@ -218,8 +225,8 @@ gst_tcp_client_src_create (GstPushSrc * psrc, GstBuffer ** outbuf)
         src->caps = caps;
       }
 
-      ret = gst_tcp_gdp_read_buffer (GST_ELEMENT (src), src->sock_fd.fd,
-          src->fdset, outbuf);
+      ret = gst_tcp_gdp_read_buffer (GST_ELEMENT (src), src->sock_fd,
+          READ_SOCKET (src), outbuf);
       break;
     default:
       /* need to assert as buf == NULL */
@@ -314,18 +321,22 @@ gst_tcp_client_src_start (GstBaseSrc * bsrc)
   gchar *ip;
   GstTCPClientSrc *src = GST_TCP_CLIENT_SRC (bsrc);
 
-  if ((src->fdset = gst_poll_new (TRUE)) == NULL)
+  /* create the control sockets before anything */
+  if (socketpair (PF_UNIX, SOCK_STREAM, 0, CONTROL_SOCKETS (src)) < 0)
     goto socket_pair;
 
+  fcntl (READ_SOCKET (src), F_SETFL, O_NONBLOCK);
+  fcntl (WRITE_SOCKET (src), F_SETFL, O_NONBLOCK);
+
   /* create receiving client socket */
   GST_DEBUG_OBJECT (src, "opening receiving client socket to %s:%d",
       src->host, src->port);
 
-  if ((src->sock_fd.fd = socket (AF_INET, SOCK_STREAM, 0)) == -1)
+  if ((src->sock_fd = socket (AF_INET, SOCK_STREAM, 0)) == -1)
     goto no_socket;
 
   GST_DEBUG_OBJECT (src, "opened receiving client socket with fd %d",
-      src->sock_fd.fd);
+      src->sock_fd);
   GST_OBJECT_FLAG_SET (src, GST_TCP_CLIENT_SRC_OPEN);
 
   /* look up name if we need to */
@@ -342,14 +353,25 @@ gst_tcp_client_src_start (GstBaseSrc * bsrc)
   g_free (ip);
 
   GST_DEBUG_OBJECT (src, "connecting to server");
-  ret = connect (src->sock_fd.fd, (struct sockaddr *) &src->server_sin,
+  ret = connect (src->sock_fd, (struct sockaddr *) &src->server_sin,
       sizeof (src->server_sin));
-  if (ret)
-    goto connect_failed;
 
-  /* add the socket to the poll */
-  gst_poll_add_fd (src->fdset, &src->sock_fd);
-  gst_poll_fd_ctl_read (src->fdset, &src->sock_fd, TRUE);
+  if (ret) {
+    gst_tcp_client_src_stop (GST_BASE_SRC (src));
+    switch (errno) {
+      case ECONNREFUSED:
+        GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+            (_("Connection to %s:%d refused."), src->host, src->port), (NULL));
+        return FALSE;
+        break;
+      default:
+        GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
+            ("connect to %s:%d failed: %s", src->host, src->port,
+                g_strerror (errno)));
+        return FALSE;
+        break;
+    }
+  }
 
   return TRUE;
 
@@ -369,22 +391,6 @@ name_resolv:
     gst_tcp_client_src_stop (GST_BASE_SRC (src));
     return FALSE;
   }
-connect_failed:
-  {
-    gst_tcp_client_src_stop (GST_BASE_SRC (src));
-    switch (errno) {
-      case ECONNREFUSED:
-        GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
-            (_("Connection to %s:%d refused."), src->host, src->port), (NULL));
-        break;
-      default:
-        GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
-            ("connect to %s:%d failed: %s", src->host, src->port,
-                g_strerror (errno)));
-        break;
-    }
-    return FALSE;
-  }
 }
 
 /* close the socket and associated resources
@@ -398,13 +404,10 @@ gst_tcp_client_src_stop (GstBaseSrc * bsrc)
   src = GST_TCP_CLIENT_SRC (bsrc);
 
   GST_DEBUG_OBJECT (src, "closing socket");
-
-  if (src->fdset != NULL) {
-    gst_poll_free (src->fdset);
-    src->fdset = NULL;
+  if (src->sock_fd != -1) {
+    close (src->sock_fd);
+    src->sock_fd = -1;
   }
-
-  gst_tcp_socket_close (&src->sock_fd);
   src->caps_received = FALSE;
   if (src->caps) {
     gst_caps_unref (src->caps);
@@ -412,6 +415,11 @@ gst_tcp_client_src_stop (GstBaseSrc * bsrc)
   }
   GST_OBJECT_FLAG_UNSET (src, GST_TCP_CLIENT_SRC_OPEN);
 
+  close (READ_SOCKET (src));
+  close (WRITE_SOCKET (src));
+  READ_SOCKET (src) = -1;
+  WRITE_SOCKET (src) = -1;
+
   return TRUE;
 }
 
@@ -421,20 +429,7 @@ gst_tcp_client_src_unlock (GstBaseSrc * bsrc)
 {
   GstTCPClientSrc *src = GST_TCP_CLIENT_SRC (bsrc);
 
-  GST_DEBUG_OBJECT (src, "set to flushing");
-  gst_poll_set_flushing (src->fdset, TRUE);
-
-  return TRUE;
-}
-
-/* will be called only between calls to start() and stop() */
-static gboolean
-gst_tcp_client_src_unlock_stop (GstBaseSrc * bsrc)
-{
-  GstTCPClientSrc *src = GST_TCP_CLIENT_SRC (bsrc);
-
-  GST_DEBUG_OBJECT (src, "unset flushing");
-  gst_poll_set_flushing (src->fdset, FALSE);
+  SEND_COMMAND (src, CONTROL_STOP);
 
   return TRUE;
 }
