@@ -28,29 +28,35 @@
  * used to interpolate frames (yet).
  * </para>
  * <para>
- * By default the element will simply negotiate the same framerate on its source and
- * sink pad and will adjust timestamps/insert/drop frames in case the input stream
- * is not respecting that framerate.
+ * By default the element will simply negotiate the same framerate on its
+ * source and sink pad and will adjust timestamps/insert/drop frames in case
+ * the input stream is not respecting that framerate.
  * </para>
  * <para>
- * A conversion to another framerate can be forced by using filtered caps on the source
- * pad.
+ * A conversion to another framerate can be forced by using filtered caps on
+ * the source pad.
  * </para>
  * <para>
- * The properties "in", "out", "duplicate" and "drop" can be read to obtain 
- * information about respectively received frame, outputed frame, duplicated frames
- * and dropped frames.
- * When the "silent" property is set to FALSE, a GObject property notification will
- * be emited whenever one of the "duplicate" or "drop" values changed. This can 
- * potentially cause performance degradation. Also note that property notification 
- * will happen in the streaming thread so applications should be prepared for this.
+ * The properties "in", "out", "duplicate" and "drop" can be read to obtain
+ * information about number of input frames, output frames, dropped frames
+ * (i.e. the number of unused input frames) and duplicated frames (i.e. the
+ *  number of times an input frame was duplicated, beside being used normally).
+ *
+ * An input stream that needs no adjustments will thus never have dropped or
+ * duplicated frames.
+ *
+ * When the "silent" property is set to FALSE, a GObject property notification
+ * will be emitted whenever one of the "duplicate" or "drop" values changes.
+ * This can potentially cause performance degradation.
+ * Note that property notification will happen from the streaming thread, so
+ * applications should be prepared for this.
  * </para>
  * <title>Example pipelines</title>
  * <para>
  * <programlisting>
  * gst-launch -v filesrc location=videotestsrc.ogg ! oggdemux ! theoradec ! videorate ! video/x-raw-yuv,framerate=15/1 ! xvimagesink
  * </programlisting>
- * Decode an Ogg/Theora and adjust the framerate to 15 fps. 
+ * Decode an Ogg/Theora file and adjust the framerate to 15 fps before playing.
  * To create the test Ogg/Theora file refer to the documentation of theoraenc.
  * </para>
  * </refsect2>
@@ -68,7 +74,7 @@ GST_DEBUG_CATEGORY (video_rate_debug);
 #define GST_CAT_DEFAULT video_rate_debug
 
 /* elementfactory information */
-static GstElementDetails video_rate_details =
+static const GstElementDetails video_rate_details =
 GST_ELEMENT_DETAILS ("Video rate adjuster",
     "Filter/Effect/Video",
     "Drops/duplicates/adjusts timestamps on video frames to make a perfect stream",
@@ -358,12 +364,8 @@ no_transform:
 static void
 gst_video_rate_reset (GstVideoRate * videorate)
 {
-  GST_DEBUG ("resetting data");
+  GST_DEBUG ("resetting internal variables");
 
-  videorate->from_rate_numerator = 0;
-  videorate->from_rate_denominator = 0;
-  videorate->to_rate_numerator = 0;
-  videorate->to_rate_denominator = 0;
   videorate->in = 0;
   videorate->out = 0;
   videorate->drop = 0;
@@ -395,6 +397,11 @@ gst_video_rate_init (GstVideoRate * videorate)
   gst_video_rate_reset (videorate);
   videorate->silent = DEFAULT_SILENT;
   videorate->new_pref = DEFAULT_NEW_PREF;
+
+  videorate->from_rate_numerator = 0;
+  videorate->from_rate_denominator = 0;
+  videorate->to_rate_numerator = 0;
+  videorate->to_rate_denominator = 0;
 }
 
 /* flush the oldest buffer */
@@ -405,10 +412,12 @@ gst_video_rate_flush_prev (GstVideoRate * videorate)
   GstBuffer *outbuf;
   GstClockTime push_ts;
 
+  if (!videorate->prevbuf)
+    goto eos_before_buffers;
+
   /* make sure we can write to the metadata */
-  outbuf =
-      gst_buffer_create_sub (videorate->prevbuf, 0,
-      GST_BUFFER_SIZE (videorate->prevbuf));
+  outbuf = gst_buffer_make_metadata_writable
+      (gst_buffer_ref (videorate->prevbuf));
 
   /* this is the timestamp we put on the buffer */
   push_ts = videorate->next_ts;
@@ -431,17 +440,15 @@ gst_video_rate_flush_prev (GstVideoRate * videorate)
       "old is best, dup, pushing buffer outgoing ts %" GST_TIME_FORMAT,
       GST_TIME_ARGS (push_ts));
 
-  if ((res = gst_pad_push (videorate->srcpad, outbuf)) != GST_FLOW_OK)
-    goto push_error;
+  res = gst_pad_push (videorate->srcpad, outbuf);
 
   return res;
 
-  /* ERRORS */
-push_error:
+  /* WARNINGS */
+eos_before_buffers:
   {
-    GST_WARNING_OBJECT (videorate, "couldn't push buffer on srcpad, reason %s",
-        gst_flow_get_name (res));
-    return res;
+    GST_INFO_OBJECT (videorate, "got EOS before any buffer was received");
+    return GST_FLOW_OK;
   }
 }
 
@@ -449,6 +456,7 @@ static void
 gst_video_rate_swap_prev (GstVideoRate * videorate, GstBuffer * buffer,
     gint64 time)
 {
+  GST_LOG_OBJECT (videorate, "swap_prev: storing buffer %p in prev", buffer);
   if (videorate->prevbuf)
     gst_buffer_unref (videorate->prevbuf);
   videorate->prevbuf = buffer;
@@ -459,6 +467,7 @@ static GstFlowReturn
 gst_video_rate_event (GstPad * pad, GstEvent * event)
 {
   GstVideoRate *videorate;
+  gboolean ret;
 
   videorate = GST_VIDEO_RATE (gst_pad_get_parent (pad));
 
@@ -489,26 +498,32 @@ gst_video_rate_event (GstPad * pad, GstEvent * event)
     }
     case GST_EVENT_EOS:
       /* flush last queued frame */
+      GST_DEBUG_OBJECT (videorate, "Got EOS");
       gst_video_rate_flush_prev (videorate);
       break;
     case GST_EVENT_FLUSH_STOP:
       /* also resets the segment */
+      GST_DEBUG_OBJECT (videorate, "Got FLUSH_STOP");
       gst_video_rate_reset (videorate);
       break;
     default:
       break;
   }
 
+  ret = gst_pad_push_event (videorate->srcpad, event);
+
 done:
   gst_object_unref (videorate);
 
-  return gst_pad_event_default (pad, event);
+  return ret;
 
   /* ERRORS */
 format_error:
   {
     GST_WARNING_OBJECT (videorate,
         "Got segment but doesn't have GST_FORMAT_TIME value");
+    gst_event_unref (event);
+    ret = FALSE;
     goto done;
   }
 }
@@ -518,21 +533,27 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
 {
   GstVideoRate *videorate;
   GstFlowReturn res = GST_FLOW_OK;
-  GstClockTime intime;
+  GstClockTime intime, in_ts;
 
   videorate = GST_VIDEO_RATE (gst_pad_get_parent (pad));
 
-  /* make sure the denominators have to be != 0 */
+  /* make sure the denominators are not 0 */
   if (videorate->from_rate_denominator == 0 ||
       videorate->to_rate_denominator == 0)
     goto not_negotiated;
 
-  intime = gst_segment_to_running_time (&videorate->segment,
-      GST_FORMAT_TIME, GST_BUFFER_TIMESTAMP (buffer));
+  in_ts = GST_BUFFER_TIMESTAMP (buffer);
 
-  /* pull in 2 buffers */
+  GST_DEBUG_OBJECT (videorate, "got buffer with timestamp %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (in_ts));
+
+  intime = gst_segment_to_running_time (&videorate->segment,
+      GST_FORMAT_TIME, in_ts);
+
+  /* we need to have two buffers to compare */
   if (videorate->prevbuf == NULL) {
     gst_video_rate_swap_prev (videorate, buffer, intime);
+    videorate->in++;
     videorate->next_ts = 0;
   } else {
     GstClockTime prevtime;
@@ -548,6 +569,20 @@ gst_video_rate_chain (GstPad * pad, GstBuffer * buffer)
 
     videorate->in++;
 
+    /* drop new buffer if it's before previous one */
+    if (intime < prevtime) {
+      GST_DEBUG_OBJECT (videorate,
+          "The new buffer (%" GST_TIME_FORMAT
+          ") is before the previous buffer (%"
+          GST_TIME_FORMAT "). Dropping new buffer.",
+          GST_TIME_ARGS (intime), GST_TIME_ARGS (prevtime));
+      videorate->drop++;
+      if (!videorate->silent)
+        g_object_notify (G_OBJECT (videorate), "drop");
+      gst_buffer_unref (buffer);
+      goto done;
+    }
+
     /* got 2 buffers, see which one is the best */
     do {
       diff1 = prevtime - videorate->next_ts;
@@ -612,6 +647,7 @@ done:
 not_negotiated:
   {
     GST_WARNING_OBJECT (videorate, "no framerate negotiated");
+    gst_buffer_unref (buffer);
     res = GST_FLOW_NOT_NEGOTIATED;
     goto done;
   }
