@@ -18,6 +18,43 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-videoscale
+ * @see_also: videorate, ffmpegcolorspace
+ *
+ * <refsect2>
+ * <para>
+ * This element resizes video frames. By default the element will try to
+ * negotiate to the same size on the source and sinkpad so that no scaling
+ * is needed. It is therefore safe to insert this element in a pipeline to
+ * get more robust behaviour without any cost if no scaling is needed.
+ * </para>
+ * <para>
+ * This element supports a wide range of color spaces including various YUV and
+ * RGB formats and is therefore generally able to operate anywhere in a
+ * pipeline.
+ * </para>
+ * <title>Example pipelines</title>
+ * <para>
+ * <programlisting>
+ * gst-launch -v filesrc location=videotestsrc.ogg ! oggdemux ! theoradec ! ffmpegcolorspace ! videoscale ! ximagesink
+ * </programlisting>
+ * Decode an Ogg/Theora and display the video using ximagesink. Since
+ * ximagesink cannot perform scaling, the video scaling will be performed by
+ * videoscale when you resize the video window.
+ * To create the test Ogg/Theora file refer to the documentation of theoraenc.
+ * </para>
+ * <para>
+ * <programlisting>
+ * gst-launch -v filesrc location=videotestsrc.ogg ! oggdemux ! theoradec ! videoscale ! video/x-raw-yuv, width=50 ! xvimagesink
+ * </programlisting>
+ * Decode an Ogg/Theora and display the video using xvimagesink with a width of
+ * 50.
+ * </para>
+ * </refsect2>
+ *
+ * Last reviewed on 2006-03-02 (0.10.4)
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -26,6 +63,7 @@
 #include <string.h>
 
 #include <gst/video/video.h>
+#include <liboil/liboil.h>
 
 #include "gstvideoscale.h"
 #include "vs_image.h"
@@ -35,12 +73,14 @@
 GST_DEBUG_CATEGORY (video_scale_debug);
 
 /* elementfactory information */
-static GstElementDetails video_scale_details =
+static const GstElementDetails video_scale_details =
 GST_ELEMENT_DETAILS ("Video scaler",
     "Filter/Effect/Video",
     "Resizes video",
     "Wim Taymans <wim.taymans@chello.be>");
 
+#define DEFAULT_PROP_METHOD	GST_VIDEO_SCALE_NEAREST
+
 enum
 {
   PROP_0,
@@ -90,7 +130,7 @@ static GType
 gst_video_scale_method_get_type (void)
 {
   static GType video_scale_method_type = 0;
-  static GEnumValue video_scale_methods[] = {
+  static const GEnumValue video_scale_methods[] = {
     {GST_VIDEO_SCALE_NEAREST, "Nearest Neighbour", "nearest-neighbour"},
     {GST_VIDEO_SCALE_BILINEAR, "Bilinear", "bilinear"},
     {0, NULL, NULL},
@@ -139,7 +179,7 @@ gst_video_scale_sink_template_factory (void)
 static void gst_video_scale_base_init (gpointer g_class);
 static void gst_video_scale_class_init (GstVideoScaleClass * klass);
 static void gst_video_scale_init (GstVideoScale * videoscale);
-static gboolean gst_video_scale_handle_src_event (GstPad * pad,
+static gboolean gst_video_scale_src_event (GstBaseTransform * trans,
     GstEvent * event);
 
 /* base transform vmethods */
@@ -214,7 +254,7 @@ gst_video_scale_class_init (GstVideoScaleClass * klass)
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_METHOD,
       g_param_spec_enum ("method", "method", "method",
-          GST_TYPE_VIDEO_SCALE_METHOD, 0, G_PARAM_READWRITE));
+          GST_TYPE_VIDEO_SCALE_METHOD, DEFAULT_PROP_METHOD, G_PARAM_READWRITE));
 
   trans_class->transform_caps =
       GST_DEBUG_FUNCPTR (gst_video_scale_transform_caps);
@@ -223,6 +263,7 @@ gst_video_scale_class_init (GstVideoScaleClass * klass)
       GST_DEBUG_FUNCPTR (gst_video_scale_get_unit_size);
   trans_class->transform = GST_DEBUG_FUNCPTR (gst_video_scale_transform);
   trans_class->fixate_caps = GST_DEBUG_FUNCPTR (gst_video_scale_fixate_caps);
+  trans_class->src_event = GST_DEBUG_FUNCPTR (gst_video_scale_src_event);
 
   trans_class->passthrough_on_same_caps = TRUE;
 
@@ -232,13 +273,9 @@ gst_video_scale_class_init (GstVideoScaleClass * klass)
 static void
 gst_video_scale_init (GstVideoScale * videoscale)
 {
-  GstBaseTransform *trans = GST_BASE_TRANSFORM (videoscale);
-
-  gst_pad_set_event_function (trans->srcpad, gst_video_scale_handle_src_event);
-
+  gst_base_transform_set_qos_enabled (GST_BASE_TRANSFORM (videoscale), TRUE);
   videoscale->tmp_buf = NULL;
-  videoscale->method = GST_VIDEO_SCALE_NEAREST;
-  /*videoscale->method = GST_VIDEO_SCALE_BILINEAR; */
+  videoscale->method = DEFAULT_PROP_METHOD;
 }
 
 
@@ -280,22 +317,42 @@ gst_video_scale_transform_caps (GstBaseTransform * trans,
 {
   GstVideoScale *videoscale;
   GstCaps *ret;
-  int i;
+  GstStructure *structure;
+  const GValue *par;
 
   videoscale = GST_VIDEO_SCALE (trans);
 
   ret = gst_caps_copy (caps);
 
-  for (i = 0; i < gst_caps_get_size (ret); i++) {
-    GstStructure *structure = gst_caps_get_structure (ret, i);
+  /* this function is always called with a simple caps */
+  g_return_val_if_fail (GST_CAPS_IS_SIMPLE (ret), NULL);
 
-    gst_structure_set (structure,
-        "width", GST_TYPE_INT_RANGE, 16, 4096,
-        "height", GST_TYPE_INT_RANGE, 16, 4096, NULL);
-    gst_structure_remove_field (structure, "pixel-aspect-ratio");
+  structure = gst_caps_get_structure (ret, 0);
+
+  gst_structure_set (structure,
+      "width", GST_TYPE_INT_RANGE, 16, 4096,
+      "height", GST_TYPE_INT_RANGE, 16, 4096, NULL);
+
+  /* if pixel aspect ratio, make a range of it */
+  if ((par = gst_structure_get_value (structure, "pixel-aspect-ratio"))) {
+    GstCaps *copy;
+    GstStructure *cstruct;
+
+    /* copy input PAR first, this is the prefered PAR */
+    gst_structure_set_value (structure, "pixel-aspect-ratio", par);
+
+    /* then make a copy with a fraction range as a second choice */
+    copy = gst_caps_copy (ret);
+    cstruct = gst_caps_get_structure (copy, 0);
+    gst_structure_set (cstruct,
+        "pixel-aspect-ratio", GST_TYPE_FRACTION_RANGE, 0, 1, G_MAXINT, 1, NULL);
+
+    /* and append */
+    gst_caps_append (ret, copy);
   }
 
   GST_DEBUG_OBJECT (trans, "returning caps: %" GST_PTR_FORMAT, ret);
+
   return ret;
 }
 
@@ -320,7 +377,7 @@ gst_video_scale_get_format (GstCaps * caps)
 
 /* calculate the size of a buffer */
 static gboolean
-gst_video_scale_prepare_size (gint format,
+gst_video_scale_prepare_size (GstVideoScale * videoscale, gint format,
     VSImage * img, gint width, gint height, guint * size)
 {
   gboolean res = TRUE;
@@ -375,12 +432,18 @@ gst_video_scale_prepare_size (gint format,
       *size = img->stride * img->height;
       break;
     default:
-      g_warning ("don't know how to scale");
-      res = FALSE;
-      break;
+      goto unknown_format;
   }
 
   return res;
+
+  /* ERRORS */
+unknown_format:
+  {
+    GST_ELEMENT_ERROR (videoscale, STREAM, NOT_IMPLEMENTED, (NULL),
+        ("Unsupported format %d", videoscale->format));
+    return FALSE;
+  }
 }
 
 static gboolean
@@ -413,15 +476,16 @@ gst_video_scale_set_caps (GstBaseTransform * trans, GstCaps * in, GstCaps * out)
   if (!ret)
     goto done;
 
-  ret = gst_video_scale_prepare_size (videoscale->format,
-      &videoscale->src, videoscale->from_width, videoscale->from_height,
-      &videoscale->src_size);
+  if (!(ret = gst_video_scale_prepare_size (videoscale, videoscale->format,
+              &videoscale->src, videoscale->from_width, videoscale->from_height,
+              &videoscale->src_size)))
+    /* prepare size has posted an error when it returns FALSE */
+    goto done;
 
-  ret &= gst_video_scale_prepare_size (videoscale->format,
-      &videoscale->dest, videoscale->to_width, videoscale->to_height,
-      &videoscale->dest_size);
-
-  if (!ret)
+  if (!(ret = gst_video_scale_prepare_size (videoscale, videoscale->format,
+              &videoscale->dest, videoscale->to_width, videoscale->to_height,
+              &videoscale->dest_size)))
+    /* prepare size has posted an error when it returns FALSE */
     goto done;
 
   if (videoscale->tmp_buf)
@@ -453,7 +517,8 @@ gst_video_scale_get_unit_size (GstBaseTransform * trans, GstCaps * caps,
   if (!parse_caps (caps, &format, &width, &height))
     return FALSE;
 
-  if (!gst_video_scale_prepare_size (format, &img, width, height, size))
+  if (!gst_video_scale_prepare_size (videoscale, format, &img, width, height,
+          size))
     return FALSE;
 
   return TRUE;
@@ -477,10 +542,28 @@ gst_video_scale_fixate_caps (GstBaseTransform * base, GstPadDirection direction,
   from_par = gst_structure_get_value (ins, "pixel-aspect-ratio");
   to_par = gst_structure_get_value (outs, "pixel-aspect-ratio");
 
+  /* we have both PAR but they might not be fixated */
   if (from_par && to_par) {
-    GValue to_ratio = { 0, };   /* w/h of output video */
-    int from_w, from_h, from_par_n, from_par_d, to_par_n, to_par_d;
-    int count = 0, w = 0, h = 0, num, den;
+    gint from_w, from_h, from_par_n, from_par_d, to_par_n, to_par_d;
+    gint count = 0, w = 0, h = 0;
+    guint num, den;
+
+    /* from_par should be fixed */
+    g_return_if_fail (gst_value_is_fixed (from_par));
+
+    from_par_n = gst_value_get_fraction_numerator (from_par);
+    from_par_d = gst_value_get_fraction_denominator (from_par);
+
+    /* fixate the out PAR */
+    if (!gst_value_is_fixed (to_par)) {
+      GST_DEBUG_OBJECT (base, "fixating to_par to %dx%d", from_par_n,
+          from_par_d);
+      gst_structure_fixate_field_nearest_fraction (outs, "pixel-aspect-ratio",
+          from_par_n, from_par_d);
+    }
+
+    to_par_n = gst_value_get_fraction_numerator (to_par);
+    to_par_d = gst_value_get_fraction_denominator (to_par);
 
     /* if both width and height are already fixed, we can't do anything
      * about it anymore */
@@ -496,16 +579,14 @@ gst_video_scale_fixate_caps (GstBaseTransform * base, GstPadDirection direction,
 
     gst_structure_get_int (ins, "width", &from_w);
     gst_structure_get_int (ins, "height", &from_h);
-    from_par_n = gst_value_get_fraction_numerator (from_par);
-    from_par_d = gst_value_get_fraction_denominator (from_par);
-    to_par_n = gst_value_get_fraction_numerator (to_par);
-    to_par_d = gst_value_get_fraction_denominator (to_par);
 
-    g_value_init (&to_ratio, GST_TYPE_FRACTION);
-    gst_value_set_fraction (&to_ratio, from_w * from_par_n * to_par_d,
-        from_h * from_par_d * to_par_n);
-    num = gst_value_get_fraction_numerator (&to_ratio);
-    den = gst_value_get_fraction_denominator (&to_ratio);
+    if (!gst_video_calculate_display_ratio (&num, &den, from_w, from_h,
+        from_par_n, from_par_d, to_par_n, to_par_d)) {
+      GST_ELEMENT_ERROR (base, CORE, NEGOTIATION, (NULL),
+          ("Error calculating the output scaled size - integer overflow"));
+      return;
+    }
+
     GST_DEBUG_OBJECT (base,
         "scaling input with %dx%d and PAR %d/%d to output PAR %d/%d",
         from_w, from_h, from_par_n, from_par_d, to_par_n, to_par_d);
@@ -516,20 +597,30 @@ gst_video_scale_fixate_caps (GstBaseTransform * base, GstPadDirection direction,
      * prefer those that have one of w/h the same as the incoming video
      * using wd / hd = num / den */
 
-    /* start with same height, because of interlaced video */
-    /* check hd / den is an integer scale factor, and scale wd with the PAR */
-    if (from_h % den == 0) {
-      GST_DEBUG_OBJECT (base, "keeping video height");
-      h = from_h;
+    /* if one of the output width or height is fixed, we work from there */
+    if (h) {
+      GST_DEBUG_OBJECT (base, "height is fixed,scaling width");
       w = h * num / den;
-    } else if (from_w % num == 0) {
-      GST_DEBUG_OBJECT (base, "keeping video width");
-      w = from_w;
+    } else if (w) {
+      GST_DEBUG_OBJECT (base, "width is fixes, scaling height");
       h = w * den / num;
     } else {
-      GST_DEBUG_OBJECT (base, "approximating but keeping video height");
-      h = from_h;
-      w = h * num / den;
+      /* none of width or height is fixed, figure out both of them based only on
+       * the input width and height */
+      /* check hd / den is an integer scale factor, and scale wd with the PAR */
+      if (from_h % den == 0) {
+        GST_DEBUG_OBJECT (base, "keeping video height");
+        h = from_h;
+        w = h * num / den;
+      } else if (from_w % num == 0) {
+        GST_DEBUG_OBJECT (base, "keeping video width");
+        w = from_w;
+        h = w * den / num;
+      } else {
+        GST_DEBUG_OBJECT (base, "approximating but keeping video height");
+        h = from_h;
+        w = h * num / den;
+      }
     }
     GST_DEBUG_OBJECT (base, "scaling to %dx%d", w, h);
 
@@ -639,7 +730,7 @@ gst_video_scale_transform (GstBaseTransform * trans, GstBuffer * in,
           vs_image_scale_nearest_RGB555 (dest, src, videoscale->tmp_buf);
           break;
         default:
-          g_warning ("don't know how to scale");
+          goto unsupported;
       }
       break;
     case GST_VIDEO_SCALE_BILINEAR:
@@ -680,29 +771,43 @@ gst_video_scale_transform (GstBaseTransform * trans, GstBuffer * in,
           vs_image_scale_linear_RGB555 (dest, src, videoscale->tmp_buf);
           break;
         default:
-          g_warning ("don't know how to scale");
+          goto unsupported;
       }
       break;
     default:
-      ret = GST_FLOW_ERROR;
-      break;
+      goto unknown_mode;
   }
 
   GST_LOG_OBJECT (videoscale, "pushing buffer of %d bytes",
       GST_BUFFER_SIZE (out));
 
   return ret;
+
+  /* ERRORS */
+unsupported:
+  {
+    GST_ELEMENT_ERROR (videoscale, STREAM, NOT_IMPLEMENTED, (NULL),
+        ("Unsupported format %d for scaling method %d",
+            videoscale->format, videoscale->method));
+    return GST_FLOW_ERROR;
+  }
+unknown_mode:
+  {
+    GST_ELEMENT_ERROR (videoscale, STREAM, NOT_IMPLEMENTED, (NULL),
+        ("Unknown scaling method %d", videoscale->method));
+    return GST_FLOW_ERROR;
+  }
 }
 
 static gboolean
-gst_video_scale_handle_src_event (GstPad * pad, GstEvent * event)
+gst_video_scale_src_event (GstBaseTransform * trans, GstEvent * event)
 {
   GstVideoScale *videoscale;
   gboolean ret;
   double a;
   GstStructure *structure;
 
-  videoscale = GST_VIDEO_SCALE (gst_pad_get_parent (pad));
+  videoscale = GST_VIDEO_SCALE (trans);
 
   GST_DEBUG_OBJECT (videoscale, "handling %s event",
       GST_EVENT_TYPE_NAME (event));
@@ -722,13 +827,13 @@ gst_video_scale_handle_src_event (GstPad * pad, GstEvent * event)
             a * videoscale->from_height / videoscale->to_height, NULL);
       }
       break;
+    case GST_EVENT_QOS:
+      break;
     default:
       break;
   }
 
-  ret = gst_pad_event_default (pad, event);
-
-  gst_object_unref (videoscale);
+  ret = GST_BASE_TRANSFORM_CLASS (parent_class)->src_event (trans, event);
 
   return ret;
 }
@@ -736,6 +841,8 @@ gst_video_scale_handle_src_event (GstPad * pad, GstEvent * event)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
+  oil_init ();
+
   if (!gst_element_register (plugin, "videoscale", GST_RANK_NONE,
           GST_TYPE_VIDEO_SCALE))
     return FALSE;
