@@ -143,7 +143,7 @@ static void gst_ximagesink_xwindow_update_geometry (GstXImageSink * ximagesink,
 static void gst_ximagesink_expose (GstXOverlay * overlay);
 
 /* ElementFactory information */
-static GstElementDetails gst_ximagesink_details =
+static const GstElementDetails gst_ximagesink_details =
 GST_ELEMENT_DETAILS ("Video sink",
     "Sink/Video",
     "A standard X based videosink",
@@ -606,7 +606,8 @@ gst_ximagesink_ximage_put (GstXImageSink * ximagesink, GstXImageBuffer * ximage)
       gst_buffer_unref (ximagesink->cur_image);
     }
     GST_LOG_OBJECT (ximagesink, "reffing %p as our current image", ximage);
-    ximagesink->cur_image = GST_XIMAGE_BUFFER (gst_buffer_ref (ximage));
+    ximagesink->cur_image =
+        GST_XIMAGE_BUFFER (gst_buffer_ref (GST_BUFFER (ximage)));
   }
 
   /* Expose sends a NULL image, we take the latest frame */
@@ -1919,7 +1920,7 @@ gst_ximagesink_class_init (GstXImageSinkClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstbasesink_class = (GstBaseSinkClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_VIDEO_SINK);
+  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->finalize = gst_ximagesink_finalize;
   gobject_class->set_property = gst_ximagesink_set_property;
@@ -2005,6 +2006,10 @@ gst_ximagesink_get_type (void)
         &navigation_info);
     g_type_add_interface_static (ximagesink_type, GST_TYPE_X_OVERLAY,
         &overlay_info);
+
+    /* register type in a more safe place instead of at runtime since the
+     * type registration is not threadsafe. */
+    gst_ximage_buffer_get_type ();
   }
 
   return ximagesink_type;
